@@ -64,17 +64,17 @@ static void my_print (char *line)
 
 
 /* --------------------- routines to handle LaTeX commands ------------- */
-int bf_fun (char *tok)
+static int bf_fun (char *tok)
 {
    return begin_parse ();
 }
 
-int em_fun (char *tok)
+static int em_fun (char *tok)
 {
    return begin_parse ();
 }
 
-int verb_fun (char *tok)
+static int verb_fun (char *tok)
 {
    int ch1, ch;
    
@@ -86,31 +86,32 @@ int verb_fun (char *tok)
    return 0;
 }
 
-int swallow_argument (char *tok)
+static int swallow_argument (char *tok)
 {
    int ch;
    while (((ch = getc (Input_Fp)) != EOF) && (ch != '{'));
    No_Output++;
    begin_parse ();
    No_Output--;
+   return 0;
 }
 
-int swallow_double_argument (char *tok)
+static int swallow_double_argument (char *tok)
 {
    swallow_argument (tok);
    return swallow_argument (tok);
 }
 
-int parbox_cmd (char *tok)
+static int parbox_cmd (char *tok)
 {
    int ch;
    while (((ch = getc (Input_Fp)) != EOF) && (ch != '{') && (ch != '['));
    if (ch == '[')
      while (((ch = getc (Input_Fp)) != EOF) && (ch != ']'));
-   swallow_argument (tok);
+   return swallow_argument (tok);
 }
 
-int section_fun (char *tok)
+static int section_fun (char *tok)
 {
    int ch;
    char line[256];
@@ -130,7 +131,7 @@ int section_fun (char *tok)
    return 0;
 }
 
-int subsection_fun (char *tok)
+static int subsection_fun (char *tok)
 {
    char line[256];
    int ch;
@@ -153,7 +154,7 @@ int subsection_fun (char *tok)
    return 0;
 }
 
-int subsubsection_fun (char *tok)
+static int subsubsection_fun (char *tok)
 {
    char line[256];
    int ch;
@@ -175,7 +176,7 @@ int subsubsection_fun (char *tok)
 }
 
 
-int extract_token (char *token)
+static int extract_token (char *token)
 {
    int ch;
    
@@ -197,7 +198,7 @@ int extract_token (char *token)
 
 
 
-int begin_fun (char *tok)
+static int begin_fun (char *tok)
 {
    int ch;
    char token[80], *t;
@@ -255,7 +256,7 @@ int begin_fun (char *tok)
    return ch;
 }
 
-int end_fun (char *tok)
+static int end_fun (char *tok)
 {
    int ch;
    while (ch = getc (Input_Fp), (ch != '{') && (ch != EOF));
@@ -273,14 +274,14 @@ int end_fun (char *tok)
    return 0;
 }
 
-int item_fun (char *tok)
+static int item_fun (char *tok)
 {
    /* Later I will worry about actually indenting it. */
    my_print ("\n  *  ");
    return 0;
 }
 
-int verbatim_fun (char *t)
+static int verbatim_fun (char *t)
 {
    int ch;
    char *verb = "\\ev";
@@ -320,7 +321,7 @@ int verbatim_fun (char *t)
 }
 
 
-int quote_fun (char *tok)
+static int quote_fun (char *tok)
 {
    int c;
    while (((c = getc (Input_Fp)) != EOF) && (c != '{'));
@@ -330,13 +331,13 @@ int quote_fun (char *tok)
    return 0;
 }
 
-int today_fun (char *tok)
+static int today_fun (char *tok)
 {
     char *the_time;
-    time_t clock;
+    time_t c;
 
-    clock = time((time_t *) 0);
-    the_time = (char *) ctime(&clock);
+    c = time((time_t *) 0);
+    the_time = (char *) ctime(&c);
    /* returns the form Sun Sep 16 01:03:52 1985\n\0 */
    the_time[24] = '\0';
    my_print (the_time);
@@ -499,7 +500,7 @@ static int alpha_token (int ch)
 }
 
 
-void non_alpha_token (int ch)
+static void non_alpha_token (int ch)
 {
    int ch1;
    
