@@ -2,12 +2,13 @@
  * take very little work to make this demo *really* useful.
  */
 
-#include <math.h>
-#include <stdio.h>
-
 #include "config.h"
 
-#ifndef NO_STDLIB_H
+#include <stdio.h>
+#include <math.h>
+#include <string.h>
+
+#ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
 
@@ -17,7 +18,7 @@
 # define USE_ANSI_COLOR
 #endif
 
-void help()
+static void help (void)
 {
    puts("ALL statements MUST be terminated with a ';' character, e.g., quit();\n");
    puts("Available functions:");
@@ -29,7 +30,7 @@ void help()
 }      
 
 /* The following three functions will be callable from the interpreter */
-void quit_calc()
+static void quit_calc (void)
 {
    SLang_reset_tty ();
 #ifdef MALLOC_DEBUG
@@ -39,12 +40,12 @@ void quit_calc()
 }
 
 
-void print(char *s)
+static void print (char *s)
 {
    fputs(s, stdout); fflush (stdout);
 }
 
-void error(char *s)
+static void error(char *s)
 {
    fprintf(stderr, "%s\n", s);
    SLang_Error = INTRINSIC_ERROR;
@@ -73,7 +74,7 @@ int main (int argc, char **argv)
 {
    if (!init_SLang() 		       /* basic interpreter functions */
        || !init_SLmath() 	       /* sin, cos, etc... */
-#ifdef unix
+#ifdef __unix__
        || !init_SLunix()
 #endif
        || !init_SLfiles()	       /* file i/o */
@@ -84,7 +85,7 @@ int main (int argc, char **argv)
 	exit(1);
      }
 
-   if (SLang_init_tty (7, 0, 1))
+   if (SLang_init_tty (-1, 0, 1))
      {
 	fprintf(stderr, "Unable to initialize tty.");
 	exit (-1);
@@ -113,7 +114,12 @@ int main (int argc, char **argv)
    
    while(1)
      {
-	if (SLang_Error) SLang_restart(1);
+	if (SLang_Error) 
+	  {
+	     SLang_doerror (NULL);
+	     SLang_restart(1);
+	  }
+	
 	SLKeyBoard_Quit = SLang_Error = 0;
 	if (take_input () && (SLang_Last_Key_Char == SLang_RL_EOF_Char))
 	  break;
@@ -172,7 +178,7 @@ static int calc_open_readline (SLang_Load_Type *x)
 	Calc_RLI.prompt = "Calc> ";
 	Calc_RLI.getkey = SLang_getkey;
 #ifdef USE_ANSI_COLOR
-	Calc_RLI.flags = SLRL_USE_ANSI;
+	Calc_RLI.flags = SL_RLINE_USE_ANSI;
 #endif
 	SLang_init_readline (&Calc_RLI);
 	return 0;
@@ -200,7 +206,7 @@ static int calc_close_readline (SLang_Load_Type *x)
 static int take_input ()
 {
    SLang_Load_Type x;
-   
+   memset ((char *) &x, 0, sizeof (SLang_Load_Type));
    x.type = 'r';		       /* readline type */
    x.read = read_using_readline;       /* function to call to perform the read */
    return SLang_load_object (&x);
