@@ -6,7 +6,7 @@
  * License or the Perl Artistic License.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 
@@ -14,14 +14,6 @@
 # include <math.h>
 #endif
 
-#ifdef HAVE_STDLIB_H
-# include <stdlib.h>
-#endif
-
-#ifdef HAVE_MALLOC_H
-# include <malloc.h>
-#endif
-
 #define SL_BYTE_COMPILING 
 
 #include "slang.h"
@@ -111,13 +103,16 @@ static int Lang_Block_Depth = -1;
 static SLBlock_Type *Lang_Object_Ptr = Lang_Interp_Stack_Static;
 /* next location for compiled obj -- points to interpreter stack initially */
 
-/* type MUST come back 0 if there is a stack underflow !!! */
+/* main_type MUST come back 0 if there is a stack underflow !!! */
 /* This routine is assumed to work even in the presence of a SLang_Error. */
 int SLang_pop(SLang_Object_Type *x)
 {
    register SLang_Object_Type *y;
    
    y = SLStack_Pointer;
+#ifdef SLANG_NOOP
+   try_again:
+#endif
    if (y == SLRun_Stack)
      {
 	x->main_type = 0;
@@ -126,12 +121,40 @@ int SLang_pop(SLang_Object_Type *x)
 	return 1;
      }
    y--;
+#ifdef SLANG_NOOP
+   if (y->main_type == SLANG_NOOP) goto try_again;
+#endif
    *x = *y;
    
    SLStack_Pointer = y;
    return(0);
 }
 
+void SLang_free_object (SLang_Object_Type *obj)
+{
+   if (obj->main_type != SLANG_DATA)
+     return;
+     
+   if (obj->sub_type == STRING_TYPE) 
+     SLFREE(obj->v.s_val);
+   else if (obj->sub_type >= ARRAY_TYPE)
+     SLang_free_user_object (obj->v.uobj);
+}
+   
+
+#ifdef SLANG_NOOP
+static void pop_noop (void)
+{
+   SLang_Object_Type *y = SLStack_Pointer;
+   
+   if (y != SLRun_Stack)
+     {
+	y--;
+	if (y->main_type == SLANG_NOOP)
+	  SLStack_Pointer = y;
+     }
+}
+#endif
 void SLang_push(SLang_Object_Type *x)
 {
    register SLang_Object_Type *y;
@@ -162,14 +185,7 @@ SLuser_Object_Type *SLang_pop_user_object (unsigned char stype)
 	return obj.v.uobj;
      }
    
-   /* Failure */
-   if (obj.main_type == SLANG_DATA)
-     {
-	if (obj.sub_type == STRING_TYPE) SLFREE(obj.v.s_val);
-	else if (obj.sub_type >= ARRAY_TYPE)
-	  SLang_free_user_object (obj.v.uobj);
-     }
-   
+   SLang_free_object (&obj);
    if (!SLang_Error) SLang_Error = TYPE_MISMATCH;
    return NULL;
 }
@@ -185,7 +201,7 @@ int SLang_pop_non_object (SLang_Object_Type *obj)
    
    if (obj->sub_type >= ARRAY_TYPE)
      {
-	SLang_free_user_object (obj->v.uobj);
+	SLang_free_object (obj);
 	SLang_Error = TYPE_MISMATCH;
 	return 1;
      }
@@ -236,8 +252,9 @@ static int lang_free_branch(SLBlock_Type *p)
 	     SLFREE (p->b.f_blk);
 	  }
 #endif
-	/* else if (type == string_type) SLFREE(p->value);
-	 This fails because objects may be attached to these strings */
+#if 0
+	else if (type == string_type) SLFREE(p->value);
+#endif
 	else if (main_type == 0) break;
 	p++;
      }
@@ -248,11 +265,11 @@ int SLang_pop_integer(int *i)
 {
    SLang_Object_Type obj;
    
-   (void) SLang_pop_non_object (&obj);
+   if (SLang_pop_non_object (&obj)) return 1;
    
    if (obj.sub_type != INT_TYPE)
      {
-	if (IS_DATA_STRING(obj)) SLFREE(obj.v.s_val);
+	SLang_free_object (&obj);
 	if (!SLang_Error) SLang_Error = TYPE_MISMATCH;
 	return(1);
      }
@@ -284,7 +301,7 @@ int SLang_pop_float(float64 *x, int *convert, int *ip)
      }
    else
      {
-	if (IS_DATA_STRING(obj)) SLFREE(obj.v.s_val);
+	SLang_free_object (&obj);
 	SLang_Error = TYPE_MISMATCH;
 	return(1);
      }
@@ -302,23 +319,33 @@ void SLang_push_float(float64 x)
 
 #endif
 
-/* if *data != 0, string should be freed upon use. */
-int SLang_pop_string(char **s, int *data)
+int SLpop_string (char **s)
 {
    SLang_Object_Type obj;
    
-   if (SLang_pop_non_object (&obj) || (obj.sub_type != STRING_TYPE))
+   if (SLang_pop (&obj))
+     return 1;
+   
+   if (obj.sub_type != STRING_TYPE)
      {
+	SLang_free_object (&obj);
 	if (!SLang_Error) SLang_Error = TYPE_MISMATCH;
+	*s = NULL;
 	return 1;
      }
    
    *s = obj.v.s_val;
-   /* return whether or not this should be freed after its use. */
-   if (obj.main_type == SLANG_DATA) *data = 1;
-   else *data = 0;
+   return 0;
+}
+
+/* if *data != 0, string should be freed upon use. */
+int SLang_pop_string(char **s, int *data)
+{
+   if (SLpop_string (s))
+     return 1;
    
-   return(0);
+   *data = 1;
+   return 0;
 }
 
 void SLang_push_integer(int i)
@@ -362,8 +389,9 @@ void SLang_push_user_object (SLuser_Object_Type *uobj)
    SLang_Object_Type obj;
    
    /* Always maintain the correct count.  It is possible for applications to
-    * switch the main_type from IVARIABLE to LANG_DATA.  For example, JED
-    * does this for its subprocess marks.
+    * switch the main_type from IVARIABLE to SLANG_DATA.  For example, JED
+    * does this for its subprocess marks.  This is done so that an intrinsic
+    * user object may be freed when it is nolonger needed.
     */
    uobj->count++;
    obj.sub_type = uobj->sub_type; obj.main_type = uobj->main_type;
@@ -374,7 +402,6 @@ void SLang_push_user_object (SLuser_Object_Type *uobj)
 
 static void call_funptr(SLang_Name_Type *);
 
-/* This is a global variable or global variable */
 static void SLang_push_variable(SLang_Object_Type *obj)
 {
    register unsigned char subtype;
@@ -388,9 +415,7 @@ static void SLang_push_variable(SLang_Object_Type *obj)
    
    if (subtype == STRING_TYPE)
      {
-	if (obj->main_type == SLANG_DATA)
-	  SLang_push_string(obj->v.s_val);
-	else SLang_push(obj);
+	SLang_push_string(obj->v.s_val);
 	return;
      }
    else if (subtype == SLANG_OBJ_TYPE)
@@ -412,21 +437,35 @@ static void SLang_push_variable(SLang_Object_Type *obj)
  *  next n items on the stack.  Some functions may require this.
  *  This returns a pointer to the last item.
  */
-SLang_Object_Type *SLreverse_stack(int *dn)
+SLang_Object_Type *_SLreverse_stack (int n)
 {
-   int n;
+#ifdef SLANG_NOOP
+   int i;
+#endif
    SLang_Object_Type *otop, *obot, tmp;
    
-   if (SLang_pop_integer(&n)) return(NULL);
-   n += *dn;
-   
    otop = SLStack_Pointer;
+#ifdef SLANG_NOOP
+   i = 0;
+   obot = otop;
+   while (i < n) 
+     {
+	obot--;
+	if (obot < SLRun_Stack)
+	  {
+	     SLang_Error = STACK_UNDERFLOW;
+	     return NULL;
+	  }
+	if (obot->main_type != SLANG_NOOP) i++;
+     }
+#else
    if ((n > otop - SLRun_Stack) || (n < 0))
      {
 	SLang_Error = STACK_UNDERFLOW;
 	return (NULL);
      }
    obot = otop - n;
+#endif
    otop--;
    while (otop > obot)
      {
@@ -513,7 +552,7 @@ long *SLang_pop_pointer(unsigned char *main_type, unsigned char *sub_type, int *
 #endif
       case INT_TYPE: val = (long *) &(p->v.i_val); break;
       case STRING_TYPE:
-	if (*main_type == SLANG_DATA) *do_free = 1;
+	*do_free = 1;
 	/* drop */
       default:
         val = (long *) p->v.s_val;
@@ -523,7 +562,7 @@ long *SLang_pop_pointer(unsigned char *main_type, unsigned char *sub_type, int *
 }
 
 
-static void lang_do_eqs(SLBlock_Type *obj)
+static void lang_do_eqs (SLBlock_Type *obj)
 {
    int y;
 #ifdef FLOAT_TYPE   
@@ -532,7 +571,6 @@ static void lang_do_eqs(SLBlock_Type *obj)
    register unsigned char sub_type;
    register SLang_Object_Type *addr;
    register long val;
-   unsigned char mtype;
    unsigned char stype;
    
    
@@ -546,34 +584,24 @@ static void lang_do_eqs(SLBlock_Type *obj)
 	val = 0;
 	addr = Local_Variable_Frame - obj->b.i_blk;
 	stype = addr->sub_type;
-	mtype = addr->main_type;
      }
 
    else if (sub_type <= SLANG_GMM)	       /* global */
      {
 	addr = (SLang_Object_Type *) obj->b.n_blk->addr;
 	val = 0;
-	mtype = addr->main_type;
 	stype = addr->sub_type;
      }
    else				       /* intrinsic */
      {
 	addr = NULL;
 	val = obj->b.n_blk->addr;      /* address of variable. */
-	mtype = obj->b.n_blk->main_type;
 	stype = obj->b.n_blk->sub_type;
      }
    
    if ((sub_type == SLANG_LEQS) || (sub_type == SLANG_GEQS))
      {
-	if (mtype == SLANG_DATA)
-	  {
-	     if (stype == STRING_TYPE) SLFREE(addr->v.s_val);
-	     else if (stype >= ARRAY_TYPE)
-	       {
-		  SLang_free_user_object (addr->v.uobj);
-	       }
-	  }
+	SLang_free_object (addr);
         SLang_pop(addr);
 	return;
      }
@@ -704,7 +732,16 @@ static void lang_do_intrinsic(SLang_Name_Type *objf)
    argc = SLANG_INTRINSIC_ARGC(*objf);
    type = SLANG_INTRINSIC_TYPE(*objf);
    
-   p7 = p6 = p5 = p4 = p3 = p2 = p1 = NULL; /* shuts up gcc, NOT needed */
+   /* These initializations are NOT needed but they shut up compilers
+    * who try to to too smart by warning about un-initialized variables.
+    * Unfortunately, the compilers are not smart enough.
+    */
+   p7 = p6 = p5 = p4 = p3 = p2 = p1 = NULL; 
+   ret = 0;
+#ifdef FLOAT_TYPE
+   xf = 0.0;
+#endif
+   
    switch (argc)
      {
 	unsigned char tmp1, tmp2;
@@ -719,7 +756,9 @@ static void lang_do_intrinsic(SLang_Name_Type *objf)
    
    /* I need to put a setjmp here so to catch any long jmps that occur
     in the user program */
-   if (!SLang_Error) switch (argc)
+   if (SLang_Error)
+     SLang_doerror (NULL);
+   else switch (argc)
      {
 	
       case 0:
@@ -823,6 +862,9 @@ static void lang_do_intrinsic(SLang_Name_Type *objf)
       case 2: if (free_p2 == 1) SLFREE(p2);
       case 1: if (free_p1 == 1) SLFREE(p1);
      }
+#ifdef SLANG_NOOP
+   pop_noop ();
+#endif
 }
 
 
@@ -984,7 +1026,7 @@ static void lang_do_ifs(register SLBlock_Type *addr)
 }
 #endif
 
-static void lang_do_else(unsigned char type, SLBlock_Type *addr, SLBlock_Type *addr_max)
+static void lang_do_else (unsigned char type, SLBlock_Type *addr, SLBlock_Type *addr_max)
 {
    int test, status;
    char *str = NULL;
@@ -993,7 +1035,8 @@ static void lang_do_else(unsigned char type, SLBlock_Type *addr, SLBlock_Type *a
    if (type == SLANG_SWITCH)
      {
 	if (SLang_pop_non_object (&cobj)) return;
-	if (IS_DATA_STRING(cobj)) str = cobj.v.s_val;
+	if (cobj.sub_type == STRING_TYPE)
+	  str = cobj.v.s_val;
      }
    
    while (addr <= addr_max)
@@ -1054,6 +1097,7 @@ void SLexecute_function(SLang_Name_Type *entry1)
    SLBlock_Type *exit_block_save;
    SLBlock_Type **user_block_save;
    SLBlock_Type *user_blocks[5];
+   char *fname;
    
    n_locals = entry->sub_type;
    
@@ -1069,6 +1113,8 @@ void SLexecute_function(SLang_Name_Type *entry1)
    
    Exit_Block_Ptr = NULL;
    
+   fname = entry->name + 1;
+   
    /* need loaded?  */
    if (n_locals == 255)
      {
@@ -1076,7 +1122,7 @@ void SLexecute_function(SLang_Name_Type *entry1)
 	n_locals = entry->sub_type;
 	if (n_locals == 255)
 	  {
-	     SLang_doerror("Function did not autoload!");
+	     _SLdo_error ("%s: Function did not autoload!", fname);
              goto the_return;
 	  }
      }
@@ -1089,7 +1135,7 @@ void SLexecute_function(SLang_Name_Type *entry1)
    i = n_locals;
    if ((lvf + i) > Local_Variable_Stack + MAX_LOCAL_STACK)
      {
-	SLang_doerror("Local Variable Stack Overflow!");
+	_SLdo_error("%s: Local Variable Stack Overflow!", fname);
 	goto the_return;
      }
    while(i--)
@@ -1099,7 +1145,7 @@ void SLexecute_function(SLang_Name_Type *entry1)
      }
    Local_Variable_Frame = lvf;
    
-   if (SLang_Enter_Function != NULL) (*SLang_Enter_Function)(entry->name + 1);
+   if (SLang_Enter_Function != NULL) (*SLang_Enter_Function)(fname);
    if (SLang_Trace)
      {
 	if ((*SLang_Trace_Function == *entry->name)
@@ -1109,7 +1155,7 @@ void SLexecute_function(SLang_Name_Type *entry1)
 	if (trace)
 	  {
 	     for (j = 0; j < trace_max; j++) buf[j] = ' ';
-	     sprintf(buf + trace_max, ">>%s\n", entry->name + 1);
+	     sprintf(buf + trace_max, ">>%s\n", fname);
 	     (*SLang_Dump_Routine)(buf);
 	     trace++;
 	  }
@@ -1120,7 +1166,7 @@ void SLexecute_function(SLang_Name_Type *entry1)
 	
 	if (trace)
 	  {
-	     sprintf(buf + trace_max, "<<%s\n", entry->name + 1);
+	     sprintf(buf + trace_max, "<<%s\n", fname);
 	     (*SLang_Dump_Routine)(buf);
 	     trace--;
 	     if (trace == 1) trace = 0;
@@ -1134,7 +1180,7 @@ void SLexecute_function(SLang_Name_Type *entry1)
      }
    
    
-   if (SLang_Exit_Function != NULL) (*SLang_Exit_Function)(entry->name + 1);
+   if (SLang_Exit_Function != NULL) (*SLang_Exit_Function)(fname);
    
    if (SLang_Error && SLang_Traceback)
      {
@@ -1144,14 +1190,7 @@ void SLexecute_function(SLang_Name_Type *entry1)
    lvf = Local_Variable_Frame;
    while(lvf > frame)
      {
-	if (lvf->main_type == SLANG_DATA)
-	  {
-	     if (lvf->sub_type == STRING_TYPE) SLFREE(lvf->v.s_val);
-	     else if (lvf->sub_type >= ARRAY_TYPE)
-	       {
-		  SLang_free_user_object (lvf->v.uobj);
-	       }
-	  }
+	SLang_free_object (lvf);
 	lvf--;
      }
    Local_Variable_Frame = lvf;
@@ -1160,6 +1199,9 @@ void SLexecute_function(SLang_Name_Type *entry1)
    Lang_Break_Condition = Lang_Return = Lang_Break = 0;
    Exit_Block_Ptr = exit_block_save;
    User_Block_Ptr = user_block_save;
+#ifdef SLANG_NOOP
+   pop_noop ();
+#endif
 }
 
 
@@ -1284,20 +1326,10 @@ static void do_binary (int op)
    SLang_Error = TYPE_MISMATCH;
    
    free_obja:
-   if (obja.main_type == SLANG_DATA)
-     {
-	if (a_sub_type == STRING_TYPE) SLFREE(obja.v.s_val);
-	else if (a_sub_type >= ARRAY_TYPE)
-	  SLang_free_user_object (obja.v.uobj);
-     }
+   SLang_free_object (&obja);
    
    free_objb:
-   if (objb.main_type == SLANG_DATA)
-     {
-	if (b_sub_type == STRING_TYPE) SLFREE(objb.v.s_val);
-	else if (b_sub_type >= ARRAY_TYPE)
-	  SLang_free_user_object (objb.v.uobj);
-     }
+   SLang_free_object (&objb);
 }
 
 static void do_unary (int op)
@@ -1327,12 +1359,7 @@ static void do_unary (int op)
      }
    else SLang_Error = TYPE_MISMATCH;
    
-   if (obj.main_type == SLANG_DATA)
-     {
-	if (sub_type == STRING_TYPE) SLFREE(obj.v.s_val);
-	else if (sub_type >= ARRAY_TYPE)
-	  SLang_free_user_object (obj.v.uobj);
-     }
+   SLang_free_object (&obj);
 }
 
 
@@ -1378,7 +1405,13 @@ static int inner_interp(SLBlock_Type *addr1)
 	  {
 	   case 0:
 	     goto end_of_switch;
-	     
+#ifdef SLANG_NOOP
+	   case SLANG_NOOP_DIRECTIVE:
+	     obj1.main_type = SLANG_NOOP;
+	     obj1.sub_type = 0;
+	     SLang_push (&obj1);
+	     break;
+#endif
 	   case SLANG_LVARIABLE:
 	     /* make val point to local stack */
 	     val =  (Local_Variable_Frame - addr->b.i_blk);
@@ -1424,11 +1457,6 @@ static int inner_interp(SLBlock_Type *addr1)
 	     bc = Lang_Break_Condition;
 	     break;
 	     
-	     /* This next one is only possible when we have a string
-	      * on a block that is not in a function.  Here just push a
-	      * copy of the string onto the stack and then later after
-	      * we have returned to top level the string will be freed.
-	      */
 	   case SLANG_DATA:
 	     SLang_push_string (addr->b.s_blk);
 	     break;
@@ -1504,7 +1532,7 @@ static int inner_interp(SLBlock_Type *addr1)
 		       int test;
 		       if (block == NULL)
 			 {
-			    SLang_doerror("Block needed.");
+			    SLang_doerror("Block needed for ELSE");
 			    break;
 			 }
 		       
@@ -1627,7 +1655,7 @@ static int inner_interp(SLBlock_Type *addr1)
 	       {
 		  inner_interp(User_Block_Ptr[addr->main_type - SLANG_X_USER0]);
 	       }
-	     else SLang_doerror("No User Block");
+	     else SLang_doerror("No block for SLANG_X_USER");
 	     bc = Lang_Break_Condition;
 	     break;
 	     
@@ -1637,7 +1665,7 @@ static int inner_interp(SLBlock_Type *addr1)
 		  inner_interp(err_block->b.blk);
 		  if (SLang_Error) err_block = NULL;
 	       }
-	     else SLang_doerror("No Error Block");
+	     else SLang_doerror("No ERROR_BLOCK");
 	     bc = Lang_Break_Condition;
 	     break;
 	     
@@ -1675,17 +1703,23 @@ static int inner_interp(SLBlock_Type *addr1)
 /* Hash value of current item to search in table */
 static unsigned char Hash;
 
-static unsigned char compute_hash(unsigned char *s)
+static unsigned char compute_hash (unsigned char *s)
 {
    register unsigned char *ss = s;
    register unsigned int h = 0;
+   
    while (*ss) h += (unsigned int) *ss++ + (h << 2);
    
    
    if (0 == (Hash = (unsigned char) h))
      {
 	Hash = (unsigned char) (h >> 8);
-	if (!Hash) Hash = *s;
+	if (!Hash) 
+	  {
+	     Hash = *s;
+	     if (Hash == 0)
+	       Hash = 1;
+	  }
      }
    
    return(Hash);
@@ -1773,6 +1807,8 @@ SLang_Name_Type *SLang_locate_name(char *name)
 {
    SLang_Name_Type *t;
    
+   if (*name == 0) return NULL;
+   
    (void) compute_hash((unsigned char *) name);
    
    t = Lang_Local_Variable_Table;
@@ -1792,7 +1828,6 @@ SLang_Name_Type *SLang_locate_name(char *name)
 static int lang_check_name(char *name)
 {
    register char *p, ch;
-   char *err = "Name Syntax";
    
    p = name;
    while ((ch = *p++) != 0)
@@ -1801,14 +1836,14 @@ static int lang_check_name(char *name)
 	if ((ch >= 'A') && (ch <= 'Z')) continue;
 	if ((ch >= '0') && (ch <= '9')) continue;
 	if ((ch == '_') || (ch == '$') || (ch == '!') || (ch == '?')) continue;
-	SLang_doerror(err);
+	_SLdo_error ("%s: bad name syntax.", name);
 	return(0);
      }
    
    p--;
    if ((int) (p - name) > SLANG_MAX_NAME_LEN)
      {
-	SLang_doerror("Name too long.");
+	_SLdo_error ("%s: Name too long.", name);
 	return(0);
      }
    return (1);
@@ -1825,7 +1860,7 @@ void SLadd_name(char *name, long addr, unsigned char main_type, unsigned char su
    if (!lang_check_name(name)) return;
    if (NULL == (entry = SLang_locate_name(name)))
      {
-	SLang_doerror("Table size exceeded!");
+	_SLdo_error ("%s: table size exceeded.", name);
 	return;  /* table full */
      }
    
@@ -2004,7 +2039,9 @@ int SLang_execute_function(char *name)
    else if (type == SLANG_INTRINSIC)
      lang_do_intrinsic(entry);
    else return(0);
-   if (SLang_Error) SLang_doerror(NULL);
+   
+   if (SLang_Error) SLang_doerror (name);
+   
    return(1);
 }
 
@@ -2113,7 +2150,7 @@ static int lang_try_binary(char *t)
    unsigned char sub, mtype;
    ssub = 0;
    
-   if (0 == (ssub = slang_eqs_name(t, SL_Binary_Ops))) return(0);
+   if (0 == (ssub = _SLeqs_name(t, SL_Binary_Ops))) return(0);
    
    if (ssub < 0)
      {
@@ -2162,7 +2199,7 @@ static void lang_begin_function(void)
 {
    if (Lang_Defining_Function || Lang_Defining_Block)
      {
-	SLang_doerror("Function nesting illegal.");
+	SLang_doerror ("Function nesting illegal.");
 	return;
      }
    
@@ -2255,13 +2292,6 @@ static void lang_begin_block(void)
 	/* SLang_doerror("Malloc error defining block."); */
 	return;
      }
-#if 0
-   if (Lang_Block_Body == (SLBlock_Type *) 0x15ae10)
-     {
-	SLang_doerror ("Here.");
-	SLang_Error = 0;
-     }
-#endif
    Lang_Block_Depth++;
    Lang_Defining_Block++;
    Lang_Object_Ptr = Lang_Block_Body;
@@ -2270,7 +2300,7 @@ static void lang_begin_block(void)
 
 
 /* see if token is a directive, and add it to current block/function */
-static Lang_Name2_Type Lang_Directives[] =
+static SLang_Name2_Type Lang_Directives[] =
 {
      {"!if", SLANG_IFNOT},
      {"if", SLANG_IF},
@@ -2304,7 +2334,7 @@ static int try_directive(char *t, int *flag)
    
    Lang_Object_Ptr->b.blk = 0;         /* not used */
    
-   if (((sub = (unsigned char) slang_eqs_name(t, Lang_Directives)) != 0)
+   if (((sub = (unsigned char) _SLeqs_name(t, Lang_Directives)) != 0)
        && *flag)
      {
 	lop = Lang_Object_Ptr - 1;
@@ -2316,9 +2346,9 @@ static int try_directive(char *t, int *flag)
 
 	     /* Lang_Object_Ptr = lop; */
 #if 1
+	     Lang_Object_Ptr->main_type = 0;
 	     if (Lang_Defining_Function || Lang_Defining_Block)
 	       return 1;
-	     Lang_Object_Ptr->main_type = 0;
 #endif
 	  }
      }
@@ -2449,7 +2479,7 @@ static int interp_variable_eqs(char *name)
 	Lang_Object_Ptr->main_type = SLANG_EQS;
 	Lang_Object_Ptr->sub_type = stype;
 	if (mtype == SLANG_LVARIABLE)
-	  Lang_Object_Ptr->b.i_blk = value;
+	  Lang_Object_Ptr->b.i_blk = (int) value;
 	else
 	  Lang_Object_Ptr->b.l_blk = value;
 	Lang_Object_Ptr++;
@@ -2461,7 +2491,7 @@ static int interp_variable_eqs(char *name)
    obj.sub_type = stype;
    
    if (mtype == SLANG_LVARIABLE)
-     obj.b.i_blk = value;
+     obj.b.i_blk = (int) value;
    else obj.b.l_blk = value;
    
    lang_do_eqs(&obj);
@@ -2670,7 +2700,7 @@ void SLadd_variable(char *name)
 	     table = (SLang_Name_Type *) SLCALLOC(MAX_LOCAL_VARIABLES, sizeof(SLang_Name_Type));
 	     if (NULL == table)
 	       {
-		  SLang_doerror("Lang: calloc error.");
+		  SLang_Error = SL_MALLOC_ERROR;
 		  return;
 	       }
 	     Lang_Local_Variable_Table = table;
@@ -2704,12 +2734,7 @@ static void interp_push_string(char *t)
    
    if (NULL == (Lang_Object_Ptr->b.s_blk = SLmake_string(t))) return;
    
-   /* a literal --- not to be freed if it is defined in a function.
-    */
-   if (Lang_Defining_Function)
-     Lang_Object_Ptr->main_type = SLANG_LITERAL;
-   else Lang_Object_Ptr->main_type = SLANG_DATA;
-   
+   Lang_Object_Ptr->main_type = SLANG_DATA;
    Lang_Object_Ptr->sub_type = STRING_TYPE;
    Lang_Object_Ptr++;
 }
@@ -2777,7 +2802,7 @@ static int try_byte_compiled(register unsigned char *s)
    if ((*s++ != '#')
        || ((n = (int) (*s++ - '0')) < 0))
      {
-	SLang_doerror("Illegal name.");
+	SLang_doerror ("Illegal name.");
 	return 1;
      }
    if (n == 0)
@@ -2836,13 +2861,13 @@ char *SLbyte_compile_name(char *name)
    
    if (SLPreprocess_Only || (*name == 0)) return name;
    
-   if (slang_eqs_name(name, Lang_Directives))
+   if (_SLeqs_name(name, Lang_Directives))
      {
 	*code = '@'; code[1] = '#';  code[2] = '0';
 	strcpy (code + 3, name);
 	return code;
      }
-   if (slang_eqs_name(name, SL_Binary_Ops))
+   if (_SLeqs_name(name, SL_Binary_Ops))
      {
 	*code = '@'; code[1] = '#';  code[2] = '1';
 	strcpy (code + 3, name);
@@ -2956,6 +2981,15 @@ void SLcompile(char *t)
 	else SLadd_variable(t);
      }
    else if (Lang_Defining_Function == -1) lang_define_function(t);
+#ifdef SLANG_NOOP
+   else if ((ch == '_') && (t[1] == '_') && !strcmp (t, "__noop__"))
+     {
+	Lang_Object_Ptr->main_type = SLANG_NOOP_DIRECTIVE;
+	Lang_Object_Ptr->sub_type = 0;
+	Lang_Object_Ptr->b.blk = NULL;         /* not used */
+	lang_try_now();
+     }
+#endif
    else if (ch == '"') interp_push_string(t);
    else if ((ch == ':') && (Lang_Defining_Block))
      {
@@ -3013,7 +3047,19 @@ void SLcompile(char *t)
 
 int SLstack_depth()
 {
+#ifdef SLANG_NOOP
+   int n = 0;
+   SLang_Object_Type *p = SLStack_Pointer;
+   
+   while (p > SLRun_Stack)
+     {
+	p--;
+	if (p->main_type != SLANG_NOOP) n++;
+     }
+   return n;
+#else
    return (int) (SLStack_Pointer - SLRun_Stack);
+#endif
 }
 
 
@@ -3025,7 +3071,7 @@ int SLstack_depth()
 
 
 /* The minus sign ones can be overloaded. */
-Lang_Name2_Type SL_Binary_Ops[] =
+SLang_Name2_Type SL_Binary_Ops[] =
 {
      {"+", -SLANG_PLUS},
      {"-", -SLANG_MINUS},
@@ -3053,7 +3099,7 @@ static char Really_Stupid_Hash[256];
 void SLstupid_hash()
 {
    register unsigned char *p;
-   register Lang_Name2_Type *d;
+   register SLang_Name2_Type *d;
    
    d = SL_Binary_Ops;
    while ((p = (unsigned char *) (d->name)) != NULL)
@@ -3072,11 +3118,11 @@ void SLstupid_hash()
 
 
 
-int slang_eqs_name(char *t, Lang_Name2_Type *d_parm)
+int _SLeqs_name(char *t, SLang_Name2_Type *d_parm)
 {
    register char *p;
    register char ch;
-   register Lang_Name2_Type *d;
+   register SLang_Name2_Type *d;
    
    ch = *t++;
    if (Really_Stupid_Hash[(unsigned char) *t] == 0) return(0);
