@@ -8,7 +8,7 @@
 
 
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -279,7 +279,7 @@ static void str_get_elem(unsigned char *s, int dat)
 
    
 
-void SLarray_putelem (void)
+static void array_putelem (int roll)
 {
    SLArray_Type *at;
    SLuser_Object_Type *uat, **uop, *uo;
@@ -303,7 +303,14 @@ void SLarray_putelem (void)
 	SLang_Error = READONLY_ERROR;
 	goto cleanup_and_return;
      }
+
+   if (roll)
+     {
+	int roll_amount = at->dim + 1;
+	SLroll_stack (&roll_amount);
+     }
    
+
    off = compute_array_offset(at);
    
    if (SLang_Error) goto cleanup_and_return;
@@ -316,10 +323,11 @@ void SLarray_putelem (void)
 	break;
 
       case STRING_TYPE:
-	if (SLang_pop_string((char **) &str, &sdat)) goto cleanup_and_return;
+	if (SLpop_string((char **) &str)) 
+	  goto cleanup_and_return;
 	
 	newstr = (unsigned char *) SLmake_string ((char *) str);
-	if (sdat == 1) SLFREE(str);
+	SLFREE(str);
 	
 	sp = (off + at->buf.s_ptr);
       
@@ -359,6 +367,16 @@ void SLarray_putelem (void)
    SLang_free_user_object (uat);
 }
 
+void SLarray_putelem (void)
+{
+   array_putelem (0);
+}
+
+void SLarray_putelem_r (void)
+{
+   array_putelem (1);
+}
+
 static void array_push_element(SLArray_Type *at, unsigned int off)
 {
    char *err_str = "Array Element uninitialized.";
@@ -461,7 +479,7 @@ void SLcopy_array (void)
    /* I might want to loosen the restriction about the actual dimensions and
     * simply demand that the size be the same */
    size = size * a->x * a->y * a->z;
-   MEMCPY ((char *)a->buf.s_ptr, (char *)b->buf.s_ptr, size);
+   SLMEMCPY ((char *)a->buf.s_ptr, (char *)b->buf.s_ptr, size);
    
    free_and_return:
    SLang_free_user_object (ua);
@@ -633,13 +651,13 @@ void SLarray_sort(char *f)
 
 void SLinit_char_array (void)
 {
-   int dat, sdat;
+   int sdat;
    SLArray_Type *at;
    SLuser_Object_Type *uat;
    unsigned char *s;
    unsigned int n, ndim;
    
-   if (SLang_pop_string((char **) &s, &dat)) return;
+   if (SLpop_string((char **) &s)) return;
    sdat = 0;			       /* No STRINGS */
    if (NULL == (uat = SLang_pop_array(&sdat))) goto free_and_return;
    at = (SLArray_Type *) uat->obj;
@@ -663,7 +681,7 @@ void SLinit_char_array (void)
    if (uat != NULL)
      SLang_free_user_object (uat);
    
-   if (dat == 1) SLFREE(s);
+   SLFREE(s);
 }
 
    
