@@ -16,9 +16,7 @@ SLang_Class_Type *SLang_Registered_Types[256];
 
 static void default_destroy (long *unused)
 {
-#ifdef msdos
    (void) unused;
-#endif
 }
 
 
@@ -157,3 +155,8 @@ void SLang_free_user_object (SLuser_Object_Type *obj)
      }
 }
 
+void SLang_free_intrinsic_user_object (SLuser_Object_Type *obj)
+{
+   obj->main_type = SLANG_DATA;
+   SLang_free_user_object (obj);
+}
