@@ -15,17 +15,15 @@ unsigned int SLang_Input_Buffer_Len = 0;
 unsigned char SLang_Input_Buffer [MAX_INPUT_BUFFER_LEN];
 
 int SLang_Abort_Char = 7;
-volatile int SLKeyBoard_Quit = 0;
 int SLang_Ignore_User_Abort = 0;
 
-
 /* This has the effect of mapping all characters in the range 128-169 to
  * ESC [ something 
  */
 #ifndef __GO32__
-#if defined(unix) || defined(vms)
-#define DEC_8BIT_HACK 64
-#endif
+# if defined(__unix__) || defined(vms)
+#  define DEC_8BIT_HACK 64
+# endif
 #endif
 
 unsigned int SLang_getkey (void)
@@ -112,6 +110,13 @@ void SLang_flush_input (void)
    
    SLang_Input_Buffer_Len = 0;
    SLKeyBoard_Quit = 0;   
-   while (SLsys_input_pending (0) > 0) (void) SLsys_getkey ();
+   while (SLsys_input_pending (0) > 0) 
+     {
+	(void) SLsys_getkey ();
+	/* Set this to 0 because SLsys_getkey may stuff keyboard buffer if
+	 * key sends key sequence (OS/2, DOS, maybe VMS).
+	 */
+	SLang_Input_Buffer_Len = 0;
+     }
    SLKeyBoard_Quit = quit;
 }
