@@ -4,7 +4,7 @@
  * You may distribute under the terms of either the GNU General Public
  * License or the Perl Artistic License.
  */
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 
@@ -12,16 +12,16 @@
 #include <stdlib.h>
 #endif
 
-#ifndef MALLOC_DEBUG
-#define MALLOC_DEBUG
+#ifdef SL_MALLOC_DEBUG
+# undef SL_MALLOC_DEBUG
 #endif
 
 #include "slang.h"
 
 #ifdef __alpha
-#define Chunk 8
+# define Chunk 8
 #else
-#define Chunk 4
+# define Chunk 4
 #endif
 
 static long Total_Allocated;
@@ -63,10 +63,8 @@ static void fixup (unsigned char *p, unsigned long n, char *what)
 #ifdef SLDEBUG_DOUT
    fprintf (dout, "ALLOC: %s\t%p %ld\n", what, p, (long) n);
 #else
-#ifdef pc_system
    (void) what;
 #endif
-#endif
 }
 
 static void SLmalloc_doerror (char *buf)
