@@ -4,10 +4,11 @@
  * You may distribute under the terms of either the GNU General Public
  * License or the Perl Artistic License.
  */
+#include "config.h"
+
 #include <stdio.h>
 #include <string.h>
 
-#include "config.h"
 #include "slang.h"
 #include "_slang.h"
 
@@ -16,7 +17,7 @@ char *SLmake_string(char *str)
    return SLmake_nstring(str, strlen (str));
 }
 
-char *SLmake_nstring (char *str, int n)
+char *SLmake_nstring (char *str, unsigned int n)
 {
    char *ptr;
    
@@ -25,7 +26,7 @@ char *SLmake_nstring (char *str, int n)
 	SLang_Error = SL_MALLOC_ERROR;
 	return(NULL);
      }
-   MEMCPY (ptr, str, n);
+   SLMEMCPY (ptr, str, n);
    ptr[n] = 0;
    return(ptr);
 }
