@@ -7,8 +7,9 @@
  */
 
 
+#define NEW_STUFF 1
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -100,15 +101,29 @@ static void directive(void);
 static void block_internal(void);
 /* static int if_fudge = 0; */
 
+static char *make_line_file_error (char *buf, int line, char *file)
+{
+   sprintf (buf, "at or before line %d: file: %s", line, file);
+   return buf;
+}
+
 static void parse_error(char *str, int flag)
 {
-   char buf[132];
+   char buf [1024];
    if (str == NULL) str = Token;
-   sprintf(buf, "Line %d: %s", LLT->n, str);
+   
+   make_line_file_error (buf, LLT->n, str);
    if (flag && SLang_Error) *buf = 0;
    SLang_doerror(buf);
 }
 
+static void do_line_file_error (int line, char *file)
+{
+   char buf[1024];
+   SLang_doerror (make_line_file_error (buf, line, file));
+}
+
+   
 
 static void function_args(void)
 {
@@ -234,7 +249,7 @@ static int try_multiple_assignment (void)
      }
    
    get_token();
-   if ((CTok == ASSIGN) || (CTok == ASSIGN_P) || (CTok == ASSIGN_P))
+   if ((CTok == ASSIGN) || (CTok == ASSIGN_P) || (CTok == ASSIGN_M))
      {
 	save_ctok = CTok;
 	get_token();
@@ -656,9 +671,12 @@ static void term(void)
 
 static void basic(void)
 {
+   int save_ctok;   
+#if !NEW_STUFF
    char *save, *save_again;
-   int count, save_ctok;
-   
+   int count;
+#endif   
+
    switch(CTok)
      {
       case CASE_TYPE:
@@ -691,11 +709,15 @@ static void basic(void)
 	      
 	   case (LEFT_P): 
 	     get_token();
+#ifdef SLANG_NOOP
+	     (*SLcompile_ptr)("__noop__");
+#endif
 	     arguments(RIGHT_P);
 	     break;
 	
 	   case BRA: 
 	     /* find end of argument list */
+#if !NEW_STUFF
 	     save = Input;
 	     count = 1;
 	     while (count)
@@ -710,12 +732,27 @@ static void basic(void)
 		  else if (CTok == BRA) count++;
 		  else if (CTok == KET) count--;
 	       }
+#else
+	     do
+	       {
+		  get_token ();
+		  expression ();
+		  if (CTok == KET) break;
+	       }
+	     while (CTok == COMMA);
 	     
+	     if (CTok != KET)
+	       {
+		  parse_error ("Expecting ']'", 1);
+		  return;
+	       }
+#endif	     
 	     get_token();
 	     if (CTok == ASSIGN)
 	       {
 		  get_token();
 		  expression();
+#if !NEW_STUFF
 		  save_again = Input;  save_ctok = CTok;
 		  Input = save;
 		  get_token();
@@ -723,12 +760,19 @@ static void basic(void)
 		  pop_name();
 		  (*SLcompile_ptr)("aput");
 		  Input = save_again;  CTok = save_ctok;
+#else
+		  pop_name();
+		  (*SLcompile_ptr)("__aput");
+#endif
+		  
 	       }
 	     else
 	       {
+#if !NEW_STUFF
 		  Input = save;
 		  get_token();
 		  arguments(KET);
+#endif
 		  pop_name();
 		  (*SLcompile_ptr)("aget");
 	       }
@@ -1008,7 +1052,7 @@ static int slang_line_ok (char *buf, int *levelp, int *exec_levelp)
 
 static SLPreprocess_Type *This_SLpp;
 
-#define STREQS(a,b) (*(a) == *(b)) && !strcmp(a,b)
+#define STREQS(a,b) ((*(a) == *(b)) && !strcmp(a,b))
 
 
 
@@ -1090,9 +1134,13 @@ static int get_token(void)
      }
    
    if (*Token == '"') return (CTok = NAME);
-   else if (0 != (type = slang_eqs_name(Token, SL_Binary_Ops)))
+   else if (0 != (type = _SLeqs_name(Token, SL_Binary_Ops)))
      {
-	return (CTok = abs(type));
+	/* Hack for dealing with && and || by C programmer. */
+	if ((Input == NULL) || (*Input != *Token))
+	  return (CTok = abs(type));
+	parse_error ("Binary op not supported.", 1);
+	return (CTok = END);
      }
 
    if (STREQS("!if", Token)) return (CTok = IF_NOT_TYPE);
@@ -1122,9 +1170,14 @@ static int get_token(void)
    if (*Token == '@')
      {
 	parse_error("Illegal Name.", 1);
-	return (CTok == END);
+	return (CTok = END);
      }
-   
+   if (STREQS("&&", Token) || STREQS("||", Token))
+     {
+	parse_error ("Binary ops `||' and `&&' not supported", 1);
+	return (CTok = END);
+     }
+
    return (CTok = NAME);
 }
 
@@ -1132,6 +1185,38 @@ static int get_token(void)
    later restored. */
 
 
+static int prep_exists_function (char *line, char comment)
+{
+   char buf[256], *b, *bmax;
+   unsigned char ch;
+   
+   bmax = buf + 255;
+   
+   while (1)
+     {
+	/* skip whitespace */
+	while ((ch = (unsigned char) *line), 
+	       ch && (ch != '\n') && (ch <= ' '))
+	  line++;
+	
+	if ((ch <= '\n') 
+	    || (ch == (unsigned char) comment)) break;
+	
+	b = buf;
+	while ((ch = (unsigned char) *line) > ' ')
+	  {
+	     if (b < bmax) *b++ = (char) ch;
+	     line++;
+	  }
+	*b = 0;
+	
+	if (SLang_is_defined (buf))
+	  return 1;
+     }
+   
+   return 0;
+}
+
 static void SLang_eval_object(SLang_Load_Type *x)
 {
    char *(*last_read_fun)(SLang_Load_Type *) = Get_Token_Read_Fun;
@@ -1142,6 +1227,9 @@ static void SLang_eval_object(SLang_Load_Type *x)
    
    last_pp = This_SLpp;
    
+   if (SLprep_exists_hook == NULL)
+     SLprep_exists_hook = prep_exists_function;
+   
    if (-1 == SLprep_open_prep (&this_pp))
      return;
    
@@ -1161,14 +1249,20 @@ static void SLang_eval_object(SLang_Load_Type *x)
    while (!SLang_Error && (CTok != END))
      {
 	LLT->top_level = 0;
-	if (CTok == EOS) 
+	if ((CTok == EOS)
+#if NEW_STUFF
+	    || (CTok == COMMA)
+#endif
+	    )
 	  {
 	     LLT->top_level = 1;
 	     get_token();
 	  }
 	else if (CTok != END) directive();
      }
-   /*   if (SLang_Error) parse_error(NULL, 0); */
+   
+   if (SLang_Error) 
+     SLang_restart (0);
    
    /* x.ptr = Input */
    
@@ -1327,10 +1421,9 @@ int SLang_load_object(SLang_Load_Type *x)
 /* Note that file could be freed from Slang during run of this routine
    so get it and store it !! (e.g., autoloading) */
       
-int SLang_load_file(char *f)
+int SLang_load_file (char *f)
 {
    SLang_Load_Type x;
-   char buf[300];
    char file[256]; 
    
    if (f != NULL) strcpy(file, f); else *file = 0;
@@ -1339,16 +1432,15 @@ int SLang_load_file(char *f)
    x.type = 'F';
    if (SL_OBJ_NOPEN == SLang_load_object(&x))
      {
-	sprintf(buf, "Error opening %s.", file);
-	SLang_doerror(buf);
+	_SLdo_error ("%s: open error", file);
+	return 0;
      }
    else if (SLang_Error) 
      {
-	sprintf(buf, "S-Lang Error: line %d: file: %s", x.n, (char *) x.name);
-	SLang_doerror(buf);
-	return(0);
+	do_line_file_error( x.n, (char *) x.name);
+	return 0;
      }
-   return(1);
+   return 1;
 }
 
 char *SLang_load_string(char *string)
@@ -1360,7 +1452,7 @@ char *SLang_load_string(char *string)
    SLang_load_object(&x);
    if (SLang_Error) 
      {
-	SLang_doerror(NULL);
+	_SLdo_error ("eval: %s", string);
      } 
    return((char *)x.name + x.ofs);
 }
@@ -1387,12 +1479,23 @@ static void SLang_byte_compile(char *s)
    n += dn;
    if (n > 250)
      {
-	fputs("\n.@", byte_compile_fp);
+	fputs("\n.", byte_compile_fp);
+	if (SLPreprocess_Only != 2)
+	  {
+	     fputc ('@', byte_compile_fp);
+	  }
 	n = dn;
      }
-   ch = (unsigned char) dn;
-   ch += 32;
+   
+   if (SLPreprocess_Only != 2)
+     {
+	ch = (unsigned char) dn;
+	ch += 32;
+     }
+   else ch = ' ';
+   
    putc((char) ch, byte_compile_fp);
+   
    fputs(s, byte_compile_fp);
    Slang_Line_Len = n + 1;
 }
@@ -1402,14 +1505,12 @@ void SLang_byte_compile_file(char *f, int *method)
    char file[256];
    SLang_Load_Type x;
    int status;
-   char buf[290];
    
    SLPreprocess_Only = *method;
    sprintf(file, "%sc", f);
    if ((byte_compile_fp = fopen(file, "w")) == NULL)
      {
-	SLang_doerror(file);
-	SLang_doerror("Error opening .slc file for byte compiling");
+	_SLdo_error("%s: unable to open", file);
 	return;
      }
    
@@ -1427,13 +1528,11 @@ void SLang_byte_compile_file(char *f, int *method)
 
    if (SL_OBJ_NOPEN == status)
      {
-	sprintf(buf, "Error opening %s for byte compiling.", f);
-	SLang_doerror(buf);
+	_SLdo_error ("%s: Error opening for byte compile.", f);
      }
    else if (SLang_Error) 
      {
-	sprintf(buf, "S-Lang Error: line %d: file: %s", x.n, (char *) x.name);
-	SLang_doerror(buf);
+	do_line_file_error (x.n, (char *) x.name);
      }
 }
 
