@@ -83,6 +83,7 @@ int SLprep_open_prep (SLPreprocess_Type *pt)
 
 void SLprep_close_prep (SLPreprocess_Type *pt)
 {
+   (void) pt;
 }
 /*}}}*/
 
@@ -143,7 +144,7 @@ static int SLwildcard (char *pattern, char *string)
 /*}}}*/
 
 /*{{{ Static variables */
-#if defined(unix) || defined(VMS)
+#if defined(__unix__) || defined(VMS)
 # define MAX_DEFINES 64
 #else
 # define MAX_DEFINES 10
