@@ -7,7 +7,7 @@
  */
 
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 
@@ -15,13 +15,7 @@
 #include "slang.h"
 #include "_slang.h"
 
-#ifndef __GO32__
-#ifdef unix
-#define real_unix
-#endif
-#endif
-
-#ifdef real_unix
+#ifdef REAL_UNIX_SYSTEM
 int SLang_RL_EOF_Char = 4;
 #else
 int SLang_RL_EOF_Char = 26;
@@ -125,7 +119,7 @@ int SLang_rline_insert (char *s)
 	*(p + n) = *p;
 	p--;
      }
-   MEMCPY ((char *) pmin, s, n);
+   SLMEMCPY ((char *) pmin, s, n);
    
    This_RLI->len += n;
    This_RLI->point += n;
@@ -226,7 +220,7 @@ static SLKeyMap_List_Type *RL_Keymap;
 
 /* This update is designed for dumb terminals.  It assumes only that the 
  * terminal can backspace via ^H, and move cursor to start of line via ^M.
- * There is a hook so the user can provide a more sophisicated update if 
+ * There is a hook so the user can provide a more sophisticated update if 
  * necessary.
  */
 
@@ -246,7 +240,7 @@ static void position_cursor (int col)
      {
 	(*This_RLI->tt_goto_column)(col);
 	This_RLI->curs_pos = col;
-	fflush(stdout);
+	fflush (stdout);
 	return;
      }
    
@@ -341,7 +335,10 @@ static void RLupdate (SLang_RLine_Info_Type *rli)
    int len, dlen, start_len = 0, prompt_len = 0, tw = 0, count;
    int want_cursor_pos;
    unsigned char *b, chb, *b_point, *p;
+   int no_echo;
    
+   no_echo = rli->flags & SL_RLINE_NO_ECHO;
+
    b_point = (unsigned char *) (rli->buf + rli->point);
    *(rli->buf + rli->len) = 0;
    
@@ -354,6 +351,9 @@ static void RLupdate (SLang_RLine_Info_Type *rli)
    b = (unsigned char *) rli->prompt;
    while (count--)
      {
+	if ((count == 0) && no_echo)
+	  break;
+
 	/* The prompt could be NULL */
 	if (b != NULL) while ((chb = *b) != 0)
 	  {
@@ -414,6 +414,9 @@ static void RLupdate (SLang_RLine_Info_Type *rli)
    len = 0;
    while (count--)
      {
+	if ((count == 0) && (no_echo))
+	  break;
+
 	while ((len < rli->edit_width) && ((chb = *b++) != 0))
 	  {
 	     dlen = Char_Widths[chb];
@@ -468,7 +471,7 @@ static int rl_eof_insert (void)
    if (This_RLI->len == 0)
      {
 	SLang_Last_Key_Char = SLang_RL_EOF_Char;
-	rl_self_insert ();
+	/* rl_self_insert (); */
 	return rl_enter ();
      }
    return 0;
@@ -498,6 +501,9 @@ int SLang_read_line (SLang_RLine_Info_Type *rli)
    rli->new_upd_len = rli->old_upd_len = 0;
    
    This_RLI->last_fun = NULL;
+   if (rli->update_hook == NULL)
+     putc ('\r', stdout);
+
    RLupdate (rli);
    
    while (1)
@@ -734,7 +740,8 @@ int SLang_init_readline (SLang_RLine_Info_Type *rli)
 	SLang_define_key1 ("^E", (VOID_STAR) rl_eol, SLKEY_F_INTRINSIC, RL_Keymap);
 	SLang_define_key1 ("^I", (VOID_STAR) rl_self_insert, SLKEY_F_INTRINSIC, RL_Keymap);
 	SLang_define_key1 ("^A", (VOID_STAR) rl_bol, SLKEY_F_INTRINSIC, RL_Keymap);
-	SLang_define_key1 ("^M", (VOID_STAR) rl_enter, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("\r", (VOID_STAR) rl_enter, SLKEY_F_INTRINSIC, RL_Keymap);
+	SLang_define_key1 ("\n", (VOID_STAR) rl_enter, SLKEY_F_INTRINSIC, RL_Keymap);
 	SLang_define_key1 ("^K", (VOID_STAR) rl_deleol, SLKEY_F_INTRINSIC, RL_Keymap);
 	SLang_define_key1 ("^L", (VOID_STAR) rl_deleol, SLKEY_F_INTRINSIC, RL_Keymap);
 	SLang_define_key1 ("^V", (VOID_STAR) rl_del, SLKEY_F_INTRINSIC, RL_Keymap);
@@ -750,10 +757,13 @@ int SLang_init_readline (SLang_RLine_Info_Type *rli)
 	SLang_define_key1 ("`", (VOID_STAR) rl_quote_insert, SLKEY_F_INTRINSIC, RL_Keymap);
 	SLang_define_key1 ("\033\\", (VOID_STAR) rl_trim, SLKEY_F_INTRINSIC, RL_Keymap);
 #else
+	SLkm_define_key  ("^C", (FVOID_STAR) rl_abort, RL_Keymap);
 	SLkm_define_key  ("^E", (FVOID_STAR) rl_eol, RL_Keymap);
+	SLkm_define_key  ("^G", (FVOID_STAR) rl_abort, RL_Keymap);
 	SLkm_define_key  ("^I", (FVOID_STAR) rl_self_insert, RL_Keymap);
 	SLkm_define_key  ("^A", (FVOID_STAR) rl_bol, RL_Keymap);
-	SLkm_define_key  ("^M", (FVOID_STAR) rl_enter, RL_Keymap);
+	SLkm_define_key  ("\r", (FVOID_STAR) rl_enter, RL_Keymap);
+	SLkm_define_key  ("\n", (FVOID_STAR) rl_enter, RL_Keymap);
 	SLkm_define_key  ("^K", (FVOID_STAR) rl_deleol, RL_Keymap);
 	SLkm_define_key  ("^L", (FVOID_STAR) rl_deleol, RL_Keymap);
 	SLkm_define_key  ("^V", (FVOID_STAR) rl_del, RL_Keymap);
@@ -778,7 +788,7 @@ int SLang_init_readline (SLang_RLine_Info_Type *rli)
    *rli->buf = 0; 
    rli->point = 0;
    
-   if (rli->flags & SLRL_USE_ANSI)
+   if (rli->flags & SL_RLINE_USE_ANSI)
      {
 	if (rli->tt_goto_column == NULL) rli->tt_goto_column = ansi_goto_column;
      }
@@ -799,20 +809,21 @@ SLang_Read_Line_Type *SLang_rline_save_line (SLang_RLine_Info_Type *rli)
 {
    SLang_Read_Line_Type *rl = NULL;
    unsigned char *buf;
-   
+
+   if ((rli == NULL) || (rli->buf == NULL))
+     return NULL;
+
    if (NULL == (rl = (SLang_Read_Line_Type *) SLMALLOC (sizeof (SLang_Read_Line_Type)))
-       || (NULL == (buf = (unsigned char *) SLMALLOC (rli->buf_len))))
+       || (NULL == (buf = (unsigned char *) SLmake_string ((char *)rli->buf))))
      {
 	if (rl != NULL) SLFREE (rl);
 	return NULL;
      }
    rl->buf = buf;
-   rl->buf_len = rli->buf_len;
+   rl->buf_len = strlen ((char *)buf);
    rl->num = rl->misc = 0;
    rl->next = rl->prev = NULL;
-   
-   strcpy ((char *) rl->buf, (char *) rli->buf);
-   
+      
    if (rli->tail != NULL) 
      {
 	rli->tail->next = rl;
