@@ -1,4 +1,6 @@
-/* Basic string functions for S-Lang */
+/* -*- mode: C; mode: fold; -*- */
+/* Standard intrinsic functions for S-Lang.  Included here are string
+   and array operations */
 /* Copyright (c) 1992, 1995 John E. Davis
  * All rights reserved.
  * 
@@ -6,16 +8,17 @@
  * License or the Perl Artistic License.
  */
 
-
-
-#include <config.h>
+#include "config.h"
+/*{{{ Include Files */
 
 #include <stdio.h>
 #include <time.h>
 
-#if defined(__GO32__) || defined(__WATCOMC__)
-# include <dos.h>
-# include <bios.h>
+#ifndef __QNX__
+# if defined(__GO32__) || defined(__WATCOMC__)
+#  include <dos.h>
+#  include <bios.h>
+# endif
 #endif
 
 #include "slang.h"
@@ -28,27 +31,20 @@
 
 #include <string.h>
 
-/* Standard intrinsic functions for S-Lang.  Included here are string
-   and array operations */
+/*}}}*/
+
       
 /* builtin stack manipulation functions */
-
-void SLdo_pop(void)
+void SLdo_pop(void) /*{{{*/
 {
    SLang_Object_Type x;
    if (SLang_pop(&x)) return;
 
-   if (x.main_type == SLANG_DATA)
-     {
-	if (x.sub_type == STRING_TYPE) SLFREE(x.v.s_val);
-	if (x.sub_type >= ARRAY_TYPE) 
-	  {
-	     SLang_free_user_object (x.v.uobj);
-	  }
-     }
+   SLang_free_object (&x);
 }
 
-static int SLdo_dup(void)
+/*}}}*/
+static int SLdo_dup(void) /*{{{*/
 {
    SLang_Object_Type x;
    if (SLang_pop(&x)) return(0);
@@ -61,8 +57,27 @@ static int SLdo_dup(void)
      }
    return(1);
 }
+
+/*}}}*/
+
+/*{{{ Utility Functions */
+
+static char Utility_Char_Table [256];
+static void set_utility_char_table (char *pos) /*{{{*/
+{
+   register char *t = Utility_Char_Table, *tmax;
+   register unsigned char ch;
+
+   tmax = t + 256;
+   while (t < tmax) *t++ = 0;
    
-static int generic_equals (void)
+   t = Utility_Char_Table;
+   while ((ch = (unsigned char) *pos++) != 0) t[ch] = 1;
+}
+
+/*}}}*/
+
+static int generic_equals (void) /*{{{*/
 {
    SLang_Object_Type obj1, obj2;
    int ret = 0;
@@ -86,79 +101,61 @@ static int generic_equals (void)
       default:
 	ret = obj1.v.l_val == obj2.v.l_val;
      }
-   if ((obj1.sub_type == STRING_TYPE) && (obj1.main_type == SLANG_DATA))
+
+   if (obj1.sub_type == STRING_TYPE)
      SLFREE (obj1.v.s_val);
-   if ((obj2.sub_type == STRING_TYPE) && (obj2.main_type == SLANG_DATA))
+   if (obj2.sub_type == STRING_TYPE)
      SLFREE (obj2.v.s_val);
    return ret;
 }
 
-static int pop_two_strings (char **a, char **b, int *free_a, int *free_b)
+/*}}}*/
+static int pop_two_strings (char **a, char **b) /*{{{*/
 {
-   if (SLang_pop_string (b, free_b)) return 1;
-   if (SLang_pop_string (a, free_a))
+   if (SLpop_string (b)) return 1;
+   if (SLpop_string (a))
      {
-	if (*free_b) SLFREE (*b);
+	SLFREE (b);
 	return 1;
      }
    return 0;
 }
 
+/*}}}*/
 
-static void SLdo_strcat(void)
+static void reverse_stack (int *count)
+{
+   (void) _SLreverse_stack (*count);
+}
+
+/*}}}*/
+
+static void SLdo_strcat(void) /*{{{*/
 {
    char *a, *b, *c;
    unsigned int len, lena;
-   int adata, bdata;
    
-   if (pop_two_strings (&a, &b, &adata, &bdata)) return;
+   if (pop_two_strings (&a, &b)) return;
 
    lena = strlen(a);
    len = lena + strlen(b) + 1;
-   if (adata == 1)
+   
+   if ((NULL == (c = (char *) SLREALLOC(a, len))))
      {
-	if ((NULL != (c = (char *) SLREALLOC(a, len))))
-	  {
-	     strcpy (c + lena, b);
-	     adata = 0;
-	  }
-	else
-	  {
-	     SLang_Error = SL_MALLOC_ERROR;
-	     return;
-	  }
-     }
-   else if (NULL != (c = (char *) SLMALLOC(len)))
-     {
-	strcpy(c, a);
-	strcpy(c + lena, b);
-     }
-   else 
-     {
-	SLang_Error = SL_MALLOC_ERROR; /* SLang_doerror("Lang Malloc error."); */
+	SLang_Error = SL_MALLOC_ERROR;
 	return;
      }
    
-   /* instead of going throug push string, push it directly */
-   SLang_push_malloced_string(c);
-   if (adata == 1) SLFREE(a);
-   if (bdata == 1) SLFREE(b);
-}
+   strcpy (c + lena, b);
 
-static char Utility_Char_Table [256];
-static void set_utility_char_table (char *pos)
-{
-   register char *t = Utility_Char_Table, *tmax;
-   register unsigned char ch;
-
-   tmax = t + 256;
-   while (t < tmax) *t++ = 0;
+   SLang_push_malloced_string (c);
    
-   t = Utility_Char_Table;
-   while ((ch = (unsigned char) *pos++) != 0) t[ch] = 1;
+   SLFREE(b);
 }
 
-static int do_trim (char **beg, char **end, char *white)
+/*}}}*/
+
+static int do_trim (char **beg, char **end, char *white) /*{{{*/
 {
    int len;
    char *a, *b;
@@ -178,46 +175,37 @@ static int do_trim (char **beg, char **end, char *white)
    return len;
 }
 
+/*}}}*/
 
-static void SLdo_strtrim(void)
+static void SLdo_strtrim(void) /*{{{*/
 {
-   char *a, *beg, *end, *c, *neew;
-   unsigned int len;
-   int adata;
+   char *a, *beg, *end;
 
-   if (SLang_pop_string(&a, &adata)) return;
+   if (SLpop_string(&a)) return;
 
    beg = a;
-   len = do_trim (&beg, &end, " \t\n");
-   
-   /* instead of going throug push string, push it directly.  This is because
-    * the string that is being copied might be a constant. */
-   if (NULL != (c = (char *) SLMALLOC(len + 1)))
-     {
-	neew = c;
-	while (beg < end) *c++ = *beg++;
-	*c = 0;
+   (void) do_trim (&beg, &end, " \t\n");
 
-	SLang_push_malloced_string(neew);
-     }
-   else SLang_Error = SL_MALLOC_ERROR; /* SLang_doerror("Lang Malloc error."); */
+   /* Since a is a malloced string that will be freed, massage it. */
+   *end = 0;
+   SLang_push_string (beg);
    
-   if (adata == 1) SLFREE(a);
+   SLFREE(a);
 }
 
+/*}}}*/
 
-static void SLdo_strcompress (void)
+static void SLdo_strcompress (void) /*{{{*/
 {
    char *str, *white, *c;
    unsigned char *s, *beg, *end;
    unsigned int len;
-   int strdata, whitedata;
 
-   if (pop_two_strings (&str, &white, &strdata, &whitedata)) return;
+   if (pop_two_strings (&str, &white)) return;
    
    beg = (unsigned char *) str;
    (void) do_trim ((char **) &beg, (char **) &end, white);
-   if (whitedata) SLFREE (white);
+   SLFREE (white);
 
    /* Determine the effective length */
    len = 0;
@@ -250,27 +238,28 @@ static void SLdo_strcompress (void)
      }
    else SLang_Error = SL_MALLOC_ERROR; /* SLang_doerror("Lang Malloc error."); */
    
-   if (strdata == 1) SLFREE(str);
+   SLFREE(str);
 }
 
+/*}}}*/
 
-static int str_replace (void)
+static int str_replace (void) /*{{{*/
 {
    char *orig, *rep, *s, *newstr, *match;
-   int free_orig, free_rep, free_match, ret;
+   int ret;
    unsigned int rep_len, match_len, new_len;
    
-   if (SLang_pop_string (&rep, &free_rep)) return 0;
-   if (pop_two_strings (&orig, &match, &free_orig, &free_match))
+   if (SLpop_string (&rep)) return 0;
+   if (pop_two_strings (&orig, &match))
      {
-	if (free_rep) SLFREE (rep);
+	SLFREE (rep);
 	return 0;
      }
    
    new_len = strlen (orig);
-   newstr = SLmake_nstring (orig, new_len);
    
-   if ((newstr != NULL) && (NULL != (s = strstr (orig, match))))
+   if ((NULL != (s = strstr (orig, match)))
+       && (NULL != (newstr = SLmake_nstring (orig, new_len))))
      {
 	match_len = strlen (match);
 	rep_len = strlen (rep);
@@ -285,23 +274,25 @@ static int str_replace (void)
 	     char *s1 = newstr + (int) (s - orig);
 	     
 	     strcpy (s1 + rep_len, s + match_len);
-	     MEMCPY (s1, rep, rep_len);
+	     SLMEMCPY (s1, rep, rep_len);
 	     SLang_push_malloced_string (newstr);
 	  }
 	ret = 1;
      }
    else ret = 0;
-   if (free_orig) SLFREE (orig);
-   if (free_match) SLFREE (match);
-   if (free_rep) SLFREE (rep);
+   
+   SLFREE (orig);
+   SLFREE (match);
+   SLFREE (rep);
    return ret;
 }
 
+/*}}}*/
 
 /* This routine returns the string with text removed between single character
    comment delimiters from the set b and e. */
 
-static void uncomment_string (char *str, char *b, char *e)
+static void uncomment_string (char *str, char *b, char *e) /*{{{*/
 {
    unsigned char chb, che;
    unsigned char *s, *cbeg, *mark;
@@ -344,12 +335,11 @@ static void uncomment_string (char *str, char *b, char *e)
    SLang_push_malloced_string (str);
 }
 
-   
-   
-static void SLquote_string (void)
+/*}}}*/
+
+static void SLquote_string (void) /*{{{*/
 {
    char *str, *quotes, *q;
-   int sdata, qdata;
    int slash;
    unsigned int len;
    register char *t, *s, *q1;
@@ -362,7 +352,7 @@ static void SLquote_string (void)
 	return;
      }
    
-   if (pop_two_strings (&str, &quotes, &sdata, &qdata)) return;
+   if (pop_two_strings (&str, &quotes)) return;
    
    /* setup the utility table to have 1s at quote char postitions. */
    set_utility_char_table (quotes);
@@ -389,35 +379,37 @@ static void SLquote_string (void)
      }
    else SLang_Error = SL_MALLOC_ERROR;
    
-   if (qdata == 1) SLFREE (quotes);
-   if (sdata == 1) SLFREE (str);
+   SLFREE (quotes);
+   SLFREE (str);
 }
 
+/*}}}*/
 
 /* returns the position of substrin in a string or null */
-static void SLdo_issubstr(void)
+static void SLdo_issubstr(void) /*{{{*/
 {
    char *a, *b, *c;
-   int adata, bdata, n;
+   int n;
 
-   if (pop_two_strings (&a, &b, &adata, &bdata)) return;
+   if (pop_two_strings (&a, &b)) return;
 
    if (NULL == (c = (char *) strstr(a, b))) n = 0; else n = 1 + (int) (c - a);
 
-   if (adata == 1) SLFREE(a);
-   if (bdata == 1) SLFREE(b);
+   SLFREE(a);
+   SLFREE(b);
    SLang_push_integer (n);
 }
 
+/*}}}*/
+
 /* returns to stack string at pos n to n + m of a */
-static void SLdo_substr(void)
+static void SLdo_substr(void) /*{{{*/
 {
-   char *a;
-   int adata, n, m;
-   char *b;
+   char *a, *b;
+   int n, m;
    int lena;
 
-   if (SLang_pop_integer(&m) || SLang_pop_integer(&n) || (SLang_pop_string(&a, &adata))) return;
+   if (SLang_pop_integer(&m) || SLang_pop_integer(&n) || (SLpop_string(&a))) return;
    lena = strlen (a);
    if (n > lena) n = lena + 1;
    if (n < 1)
@@ -427,45 +419,51 @@ static void SLdo_substr(void)
    else
      {
 	n--;
-	if (m < 0) m = 0; else if (n + m > lena) m = lena - n;
+	if (m < 0) m = lena; 
+	if (n + m > lena) m = lena - n;
 	b = SLmake_nstring (a + n, (unsigned int) m);
 	if (b != NULL)
 	  SLang_push_malloced_string (b);
      }
-   if (adata == 1) SLFREE(a);
+   SLFREE(a);
 }
+
+/*}}}*/
+
 /* substitute char m at positin string n in string*/
-static void SLdo_strsub(void)
+static void SLdo_strsub(void) /*{{{*/
 {
-   char *a, *b;
-   int adata, n, m;
+   char *a;
+   int n, m;
    unsigned int lena;
    
-   if (SLang_pop_integer(&m) || SLang_pop_integer(&n) || (SLang_pop_string(&a, &adata))) return;
+   if (SLang_pop_integer(&m) || SLang_pop_integer(&n) || (SLpop_string(&a))) 
+     return;
+   
    lena = strlen (a);
    
-   if ((lena < n) || (n <= 0))
+   if ((n <= 0) || (lena < (unsigned int) n))
      {
 	SLang_Error = SL_INVALID_PARM;
+	SLFREE(a);
+	return;
      }
-   else if (NULL != (b = SLmake_nstring (a, lena)))
-     {
-	b[n - 1] = (char) m;
-	SLang_push_malloced_string (b);
-     }
-   if (adata) SLFREE(a);
+   
+   a[n - 1] = (char) m;
+   SLang_push_malloced_string (a);
 }
 
-static void SLdo_strup(void)
+/*}}}*/
+
+static void SLdo_strup(void) /*{{{*/
 {
    unsigned char c, *a;
-   int adata;
 
-   if (SLang_pop_string((char **) &a, &adata)) return;
-   SLang_push_string((char *) a);
-   if (adata) SLFREE(a);
+   if (SLpop_string((char **) &a)) 
+     return;
+   
+   SLang_push_malloced_string ((char *) a);
    
-   a = (unsigned char *) (SLStack_Pointer - 1)->v.s_val;
    while ((c = *a) != 0)
      {
 	/* if ((*a >= 'a') && (*a <= 'z')) *a -= 32; */
@@ -474,7 +472,9 @@ static void SLdo_strup(void)
      }
 }
 
-static int do_upper (void)
+/*}}}*/
+
+static int do_upper (void) /*{{{*/
 {
    int ch;
    
@@ -482,24 +482,26 @@ static int do_upper (void)
    return UPPER_CASE(ch);
 }
 
-static int do_lower (void)
+/*}}}*/
+
+static int do_lower (void) /*{{{*/
 {
    int ch;
    
    if (SLang_pop_integer (&ch)) return -1;
    return LOWER_CASE(ch);
 }
-   
-static void SLdo_strlow(void)
+
+/*}}}*/
+
+static void SLdo_strlow(void) /*{{{*/
 {
    unsigned char c, *a;
-   int adata;
 
-   if (SLang_pop_string((char **) &a, &adata)) return;
-   SLang_push_string((char *) a);
-   if (adata) SLFREE(a);
-   a = (unsigned char *) (SLStack_Pointer - 1)->v.s_val;
+   if (SLpop_string((char **) &a)) return;
    
+   SLang_push_malloced_string ((char *) a);
+
    while ((c = *a) != 0)
      {
 	/* if ((*a >= 'a') && (*a <= 'z')) *a -= 32; */
@@ -508,9 +510,11 @@ static void SLdo_strlow(void)
      }
 }
 
-static int do_strchop (void)
+/*}}}*/
+
+static int do_strchop (void) /*{{{*/
 {
-   int delim, quote, do_free, count;
+   int delim, quote, count;
    char *str, *s0, *elm;
    register char *s1;
    register unsigned char ch;
@@ -518,14 +522,14 @@ static int do_strchop (void)
 
    if (SLang_pop_integer (&quote)
        || (SLang_pop_integer (&delim))
-       || (SLang_pop_string (&str, &do_free)))
+       || (SLpop_string (&str)))
      return 0;
    
    if ((quote < 0) || (quote > 255) 
        || (delim <= 0) || (delim > 255))
      {
 	SLang_Error = TYPE_MISMATCH;
-	if (do_free) SLFREE (str);
+	SLFREE (str);
 	return 0;
      }
    
@@ -575,7 +579,7 @@ static int do_strchop (void)
 	else s1++;
      }
 
-   if (do_free) SLFREE (str);
+   SLFREE (str);
    if (SLang_Error)
      {
 	while (count != 0) 
@@ -588,39 +592,41 @@ static int do_strchop (void)
    return count;
 }
 
-static int do_strchopr (void)
+/*}}}*/
+
+static int do_strchopr (void) /*{{{*/
 {
    int count;
-   int zero = 0;
    
    count = do_strchop ();
    if (count <= 0) return count;
    
-   /* What was I thinking when I wrote SLreverse_stack !!! */
-   SLang_push_integer (count);
-   (void) SLreverse_stack (&zero);
+   _SLreverse_stack (count);
    return count;
 }
 
-static void SLdo_strcmp(void)
+/*}}}*/
+
+static void SLdo_strcmp(void) /*{{{*/
 {
    char *a, *b;
-   int adata, bdata;
 
-   if (pop_two_strings (&a, &b, &adata, &bdata)) return;
+   if (pop_two_strings (&a, &b)) return;
 
    SLang_push_integer (strcmp(a, b));
 
-   if (adata) SLFREE(a);
-   if (bdata) SLFREE(b);
+   SLFREE(a);
+   SLFREE(b);
 }
 
-static void SLdo_strncmp(void)
+/*}}}*/
+
+static void SLdo_strncmp(void) /*{{{*/
 {
    char *a, *b;
-   int adata, bdata, n;
+   int n;
    
-   if (SLang_pop_integer(&n) || pop_two_strings (&a, &b, &adata, &bdata))
+   if (SLang_pop_integer(&n) || pop_two_strings (&a, &b))
      return;
    
    if (n < 0)
@@ -630,29 +636,34 @@ static void SLdo_strncmp(void)
    else
      SLang_push_integer (strncmp(a, b, (unsigned int) n));
 
-   if (adata) SLFREE(a);
-   if (bdata) SLFREE(b);
+   SLFREE(a);
+   SLFREE(b);
 }
 
-static void SLdo_strlen(void)
+/*}}}*/
+
+static void SLdo_strlen(void) /*{{{*/
 {
    char *a;
-   int adata;
 
-   if (SLang_pop_string(&a, &adata)) return;
+   if (SLpop_string(&a)) return;
 
    SLang_push_integer ((int) strlen(a));
 
-   if (adata) SLFREE(a);
+   SLFREE(a);
 }
 
-static int SLdo_isdigit(char *what)
+/*}}}*/
+
+static int SLdo_isdigit(char *what) /*{{{*/
 {
    if ((*what >= '0') && (*what <= '9')) return(1); else return(0);
 }
 
+/*}}}*/
+
 /* convert object to integer form */
-static void SLdo_int(void)
+static void SLdo_int (void) /*{{{*/
 {
    SLang_Object_Type x;
    int i;
@@ -666,13 +677,12 @@ static void SLdo_int(void)
 	SLang_push(&x);
 	return;
      }
-
    else if (stype == STRING_TYPE)
      {
 	stype = x.main_type;
 	i = (int) *(unsigned char *) x.v.s_val;
 	
-	if (stype == SLANG_DATA) SLFREE(x.v.s_val);
+	SLFREE(x.v.s_val);
      }
 #ifdef FLOAT_TYPE
    else if (stype == FLOAT_TYPE)
@@ -688,24 +698,27 @@ static void SLdo_int(void)
    SLang_push_integer(i);
 }
 
+/*}}}*/
+
 static char Float_Format[16] = "%f";
 
-static void set_float_format (char *s)
+static void set_float_format (char *s) /*{{{*/
 {
    strncpy (Float_Format, s, 15);
    Float_Format[15] = 0;
 }
 
+/*}}}*/
+
 
 /* Conver string to integer */
-static void SLdo_integer(void)
+static void SLdo_integer(void) /*{{{*/
 {
    char *a;
-   int adata;
    int i;
    
    if (SLang_Error) return;
-   if (SLang_pop_string(&a, &adata)) return;
+   if (SLpop_string(&a)) return;
    
    i = SLatoi((unsigned char *) a);
    if (SLang_Error)
@@ -714,11 +727,13 @@ static void SLdo_integer(void)
 	SLang_doerror ("The string cannot be converted to an integer.");
      }
    else SLang_push_integer (i);
-   if (adata) SLFREE(a);
+   SLFREE(a);
 }
 
+/*}}}*/
+
 /* convert integer to a sring of length 1 */
-static void SLdo_char(void)
+static void SLdo_char(void) /*{{{*/
 {
    char ch, buf[2];
    int x;
@@ -731,8 +746,10 @@ static void SLdo_char(void)
    SLang_push_string((char *) buf);
 }
 
+/*}}}*/
+
 /* format object into a string */
-char *SLstringize_object (SLang_Object_Type *obj)
+char *SLstringize_object (SLang_Object_Type *obj) /*{{{*/
 {
    unsigned char stype;
    char *s;
@@ -769,23 +786,22 @@ char *SLstringize_object (SLang_Object_Type *obj)
    return SLmake_string (s);
 }
 
-static void SLdo_string(void)
+/*}}}*/
+
+static void SLdo_string(void) /*{{{*/
 {
    SLang_Object_Type x;
    
    if (SLang_pop (&x)) return;
    SLang_push_malloced_string (SLstringize_object (&x));
    
-   if (x.main_type == SLANG_DATA)
-     {
-	if (x.sub_type == STRING_TYPE) SLFREE(x.v.s_val);
-	else if (x.sub_type >= ARRAY_TYPE)
-	  SLang_free_user_object (x.v.uobj);
-     }
+   SLang_free_object (&x);
 }
 
+/*}}}*/
+
 /* probably more useful to have a argc, argv thing */
-int SLang_run_hooks(char *hook, char *opt1, char *opt2)
+int SLang_run_hooks(char *hook, char *opt1, char *opt2) /*{{{*/
 {
    int ret = 0;
 
@@ -796,34 +812,41 @@ int SLang_run_hooks(char *hook, char *opt1, char *opt2)
    return (ret && !SLang_Error);
 }
 
-static void lang_getenv_cmd(char *s)
+/*}}}*/
+
+static void lang_getenv_cmd(char *s) /*{{{*/
 {
    char *t;
    if (NULL == (t = getenv(s))) t = "";
    SLang_push_string(t);
 }
 
+/*}}}*/
+
 #ifdef HAVE_PUTENV
-static void lang_putenv_cmd(void)
+static void lang_putenv_cmd(void) /*{{{*/
 {
    char *s;
-   int d;
-   if (SLang_pop_string(&s, &d)) return;
+
+   if (SLpop_string(&s)) return;
 
    if (putenv (s))
      {
 	SLang_Error = INTRINSIC_ERROR;
-	if (d == 1) SLFREE (s);
+	SLFREE (s);
      }
    
    /* Note that s is NOT freed */
 }
+
+/*}}}*/
+
 #endif
-static int lang_apropos1(char *s, SLang_Name_Type *table, int max)
+
+static int lang_apropos1(char *s, SLang_Name_Type *table, int max) /*{{{*/
 {
    int all = 0, n = 0;
    char *nm;
-   SLang_Object_Type obj;
 
    if (*s == 0) all = 1;
 
@@ -833,11 +856,8 @@ static int lang_apropos1(char *s, SLang_Name_Type *table, int max)
 	if ((*nm != 1) && (all || (NULL != strstr(nm, s))))
 	  {
 	     n++;
-	     /* since string is static, make it literal */
-	     obj.main_type = SLANG_LITERAL; obj.sub_type = STRING_TYPE;
-	     obj.v.s_val = nm;
-	     SLang_push(&obj);
-	     if (SLang_Error) return(1);
+	     SLang_push_string (nm);
+	     if (SLang_Error) return 1;
 	  }
 	table++;
 	max--;
@@ -845,7 +865,9 @@ static int lang_apropos1(char *s, SLang_Name_Type *table, int max)
    return n;
 }
 
-static void lang_apropos(char *s)
+/*}}}*/
+
+static void lang_apropos(char *s) /*{{{*/
 {
    int n;
    SLName_Table *nt;
@@ -860,7 +882,21 @@ static void lang_apropos(char *s)
    SLang_push_integer(n);
 }
 
-static void lang_print_stack (void)
+/*}}}*/
+
+static int get_object_type (void) /*{{{*/
+{
+   SLang_Object_Type obj;
+   
+   if (SLang_pop (&obj))
+     return -1;
+   SLang_push (&obj);
+   return obj.sub_type;
+}
+
+/*}}}*/
+
+static void lang_print_stack (void) /*{{{*/
 {
    SLang_Object_Type *x = SLStack_Pointer;
    int n;
@@ -901,10 +937,11 @@ static void lang_print_stack (void)
      }
 }
 
+/*}}}*/
 
 /* sprintf functionality for S-Lang */
 
-static char *SLdo_sprintf(char *fmt)
+static char *SLdo_sprintf(char *fmt) /*{{{*/
 {
    register char *p = fmt, ch;
    char *out = NULL, *outp = NULL;
@@ -923,9 +960,14 @@ static char *SLdo_sprintf(char *fmt)
    
    while (1)
      {
-	while (ch = *p, ch && (ch != '%')) p++;
-		  
-	/* p points at '%' */
+	while ((ch = *p) != 0)
+	  {
+	     if (ch == '%')
+	       break;
+	     p++;
+	  }
+	
+	/* p points at '%' or 0 */
 	
 	dlen = (unsigned int) (p - fmt);
 	
@@ -1037,17 +1079,20 @@ static char *SLdo_sprintf(char *fmt)
 	switch(ch)
 	  {
 	     case 's': 
-	     if (SLang_pop_string((char **) &varp, &do_free)) return (out);
+	     if (SLpop_string((char **) &varp)) return (out);
+	     do_free = 1;
 	     guess_size = strlen((char *) varp);
 	     use_varp = 1;
 	     break;
-	     
+
+#if 1
 	   case '%': 
 	     guess_size = 1;
 	     do_free = 0;
 	     use_varp = 1;
 	     varp = (long *) "%";
 	     break;
+#endif
 	     
 	     case 'c': guess_size = 1;
 	     /* drop */
@@ -1107,7 +1152,7 @@ static char *SLdo_sprintf(char *fmt)
 	if (use_varp)
 	  {
 	     sprintf(outp, dfmt, varp);
-	     if (do_free == 1) SLFREE(varp);
+	     if (do_free) SLFREE(varp);
 	  }
 #ifdef FLOAT_TYPE
 	else if (use_float) sprintf(outp, dfmt, x);
@@ -1127,20 +1172,22 @@ static char *SLdo_sprintf(char *fmt)
    
    return (out);
 }
-   
 
+/*}}}*/
 
-static void SLsprintf(void)
+static void SLsprintf(void) /*{{{*/
 {
    register char *p, ch, *b;
    char buf[256], ch1, *fmt;
-   int n = 1, do_free;
+   int n = 1;
    SLang_Object_Type *ptr;
-
-   if (NULL == (ptr = SLreverse_stack(&n))) return;
-   if (SLang_pop_string(&fmt, &do_free)) return;
+   
+   if (SLang_pop_integer (&n)) return;
+   if (NULL == (ptr = _SLreverse_stack (n + 1))) return;
+   if (SLpop_string(&fmt)) return;
+   
    strncpy(buf, fmt, 255);
-   if (do_free == 1) SLFREE(fmt);
+   SLFREE(fmt);
 
    buf[255] = 0;
    p = b = buf;
@@ -1170,10 +1217,10 @@ static void SLsprintf(void)
    if (p != NULL) SLang_push_malloced_string (p);
 }
 
+/*}}}*/
 
 /* converts string s to a form that can be used in an eval */
-
-static void make_printable_string(char *s)
+static void make_printable_string(char *s) /*{{{*/
 {
    unsigned int len;
    register char *s1 = s, ch, *ss1;
@@ -1212,8 +1259,9 @@ static void make_printable_string(char *s)
    SLang_push_string(ss);
 }
 
+/*}}}*/
    
-char *SLang_extract_list_element(char *list, int *nth, int *delim)
+char *SLang_extract_list_element(char *list, int *nth, int *delim) /*{{{*/
 {
    int d = *delim, n = *nth;
    static char elem[256];
@@ -1231,7 +1279,9 @@ char *SLang_extract_list_element(char *list, int *nth, int *delim)
    return (elem);
 }
 
-static int SLang_is_list_element(char *list, char *elem, int *delim)
+/*}}}*/
+
+static int SLang_is_list_element(char *list, char *elem, int *delim) /*{{{*/
 {
    int d = *delim, n;
    unsigned int len;
@@ -1250,10 +1300,12 @@ static int SLang_is_list_element(char *list, char *elem, int *delim)
    return (0);
 }
 
+/*}}}*/
+
 /* Regular expression routines for strings */
 static SLRegexp_Type regexp_reg;
 
-static int string_match (char *str, char *pat, int *np)
+static int string_match (char *str, char *pat, int *np) /*{{{*/
 {
    int n = *np;
    unsigned int len;
@@ -1272,11 +1324,12 @@ static int string_match (char *str, char *pat, int *np)
    
    n--;
    len = strlen(str);
-   if ((n < 0) || (n >= len))
+   if ((n < 0) || ((unsigned int) n >= len))
      {
-	SLang_Error = SL_INVALID_PARM;
+	/* SLang_Error = SL_INVALID_PARM; */
 	return 0;
      }
+
    str += n;
    len -= n;
    
@@ -1288,7 +1341,9 @@ static int string_match (char *str, char *pat, int *np)
    return (1 + (int) ((char *) match - str));
 }
 
-static int string_match_nth(int *np)
+/*}}}*/
+
+static int string_match_nth(int *np) /*{{{*/
 {
    int n = *np, beg;
    
@@ -1302,15 +1357,19 @@ static int string_match_nth(int *np)
    return regexp_reg.end_matches[n];
 }
 
-static int my_system (char *s)
+/*}}}*/
+
+static int my_system (char *s) /*{{{*/
 {
    return system (s);
 }
 
+/*}}}*/
+
 #include <time.h>
 
-#if defined(__GO32__) || defined(__WATCOMC__)
-static char *djgpp_current_time (void)
+#if defined(__GO32__) || (!defined(__QNX__) && defined(__WATCOMC__))
+static char *djgpp_current_time (void) /*{{{*/
 {
    union REGS rg;
    unsigned int year;
@@ -1350,9 +1409,12 @@ static char *djgpp_current_time (void)
 	   day, hour, minute, sec, year);
    return the_date;
 }
+
+/*}}}*/
+
 #endif
 
-char *SLcurrent_time_string (void)
+char *SLcurrent_time_string (void) /*{{{*/
 {
    char *the_time;
 #ifndef __GO32__
@@ -1368,6 +1430,21 @@ char *SLcurrent_time_string (void)
    return(the_time);
 }
 
+/*}}}*/
+
+static int load_file (void) /*{{{*/
+{
+   char *s;
+   int ret;
+   
+   if (SLpop_string (&s)) return 0;
+   ret = SLang_load_file (s);
+   SLFREE (s);
+   return ret;
+}
+
+/*}}}*/
+
 /* These are here to let interpreter script have access to these values. */
 static int Integer_Type_Var = INT_TYPE;
 static int String_Type_Var = STRING_TYPE;
@@ -1375,7 +1452,7 @@ static int String_Type_Var = STRING_TYPE;
 static int Float_Type_Var = FLOAT_TYPE;
 #endif
 
-SLang_Name_Type SLang_Basic_Table[] =
+SLang_Name_Type SLang_Basic_Table[] = /*{{{*/
 {
    MAKE_INTRINSIC(".autoload",  SLang_autoload, VOID_TYPE, 2),
    /* Prototype: Void autoload(String fun, String file);
@@ -1508,7 +1585,7 @@ SLang_Name_Type SLang_Basic_Table[] =
     * Related Functions: @getenv@
     */
 #endif
-   MAKE_INTRINSIC(".evalfile",  SLang_load_file, INT_TYPE, 1),
+   MAKE_INTRINSIC(".evalfile",  load_file, INT_TYPE, 0),
    /* Prototype: Integer evalfile (String file);
     * Load @file@ as S-Lang code.  If loading is successful, a non-zero result 
     * will be returned.  If @file@ is not found, zero will be returned.
@@ -1540,9 +1617,10 @@ SLang_Name_Type SLang_Basic_Table[] =
     * Related Functions: @pop@, @copy_array@ */
    MAKE_INTRINSIC(".substr",  SLdo_substr, VOID_TYPE, 0),
    /* Prototype: String substr (String s, Integer n, Integer len);
-    * This function returns a substring with length @len@ of string @s@
-    * beginning at position @n@.  The first character of @s@ is given by @n@
-    * equal to 1.  For example, 
+    * This function returns a substring with length @len@ of string
+    * @s@ beginning at position @n@.  If @len@ is @-1@, the entire
+    * length of the string @s@ will be used for @len@.  The first
+    * character of @s@ is given by @n@ equal to 1.  For example, 
     * @ substr ("To be or not to be", 7, 5);
     * returns @"or no"@.
     * Related Functions: @is_substr@
@@ -1659,14 +1737,16 @@ SLang_Name_Type SLang_Basic_Table[] =
     * @  if (matched) {
     * @      (pos, len) = string_match_nth(2);
     * @  }
-    * This will set @matched@ to 1 since a match will be found at the first
-    * position, @pos@ to 7 since @w@ is the 7th character of the string, and
-    * len to 5 since @"world"@ is 5 characters long.
+    * This will set @matched@ to 1 since a match will be found at the
+    * first position, @pos@ to 6 since @w@ is the offset 6 characters
+    * from the beginning of the string, and len to 5 since @"world"@
+    * is 5 characters long.
     * 
-    * Please note that the position offset is affected by the value of the 
-    * offset parameter to the @string_match@ function.  For example, if the value
-    * of the last parameter to the @string_match@ function had been 3, @pos@ would
-    * have been set to 5.
+    * Please note that the position offset is NOT affected by the
+    * value of the offset parameter to the @string_match@ function.
+    * For example, if the value of the last parameter to the
+    * @string_match@ function had been 3, @pos@ would still have been set 
+    * to 6.
     */
    MAKE_VARIABLE("._traceback", &SLang_Traceback, INT_TYPE, 0),
    /* Prototype Integer _traceback;
@@ -1743,7 +1823,7 @@ SLang_Name_Type SLang_Basic_Table[] =
    
    /* array ops: */
    MAKE_INTRINSIC(".create_array",  SLang_create_array, VOID_TYPE, 0),
-   /* Prototype: create_array (Integer type, Integer i_1, i_2...i_dim, Integer dim);
+   /* Prototype: Array create_array (Integer type, Integer i_1, i_2, ..., i_dim, Integer dim);
     * Creates an array of type @type@ with dimension @dim@.
     * @i_n@ is an integer which specifies the maximum size of array in 
     * direction @n@.   @type@ is a control integer which specifies the type 
@@ -1769,6 +1849,7 @@ SLang_Name_Type SLang_Basic_Table[] =
     * syntax.
     * Related Functions: @aput@, @create_array@
     */
+   MAKE_INTRINSIC(".__aput",  SLarray_putelem_r, VOID_TYPE, 0),
    MAKE_INTRINSIC(".aput",  SLarray_putelem, VOID_TYPE, 0),
    /* Prototype: Void aput (Integer i, Integer j,..., Integer k, Array a);
     * This function sets an element of the array @a@.  It is equivalent
@@ -1903,7 +1984,22 @@ SLang_Name_Type SLang_Basic_Table[] =
     * This function returns number of items on stack prior to the call of
     * @_stkdepth@.
     * 
-    * Related Functions: @print_stack@
+    * Related Functions: @print_stack@, @_obj_type@
+    */
+   MAKE_INTRINSIC("._stk_reverse", reverse_stack, VOID_TYPE, 1),
+   /* Prototype: Void _stk_reverse (Integer n);
+    */
+   MAKE_INTRINSIC("._obj_type", get_object_type, INT_TYPE, 0),
+   /* Prototype: Integer _obj_type ();
+    * This function returns type information about the object on the top of the
+    * stack.  It does not remove the object from the stack.  The value returned
+    * in an integer with following meaning:
+    * @   2  Integer
+    * @   3  Float
+    * @   6  Pointer
+    * @  10  String
+    * @  20  Array
+    * Other values correspond to application dependent types.
     */
    MAKE_INTRINSIC(".print_stack", lang_print_stack, VOID_TYPE, 0),
    /* Prototype: Void print_stack (void);
@@ -2071,8 +2167,10 @@ SLang_Name_Type SLang_Basic_Table[] =
    SLANG_END_TABLE
 };
 
+/*}}}*/
 
-int init_SLang()
+
+int init_SLang() /*{{{*/
 {
    char name[3];
    int i;
@@ -2121,3 +2219,5 @@ int init_SLang()
    SLang_init_case_tables ();
    return (SLang_Error == 0);
 }
+
+/*}}}*/
