@@ -7,14 +7,13 @@
  */
 
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <signal.h>
 /* sequent support thanks to Kenneth Lorber <keni@oasys.dt.navy.mil> */
 /* SYSV (SYSV ISC R3.2 v3.0) provided by iain.lea@erlm.siemens.de */
 
-
 #if defined (_AIX) && !defined (_ALL_SOURCE)
 # define _ALL_SOURCE	/* so NBBY is defined in <sys/types.h> */
 #endif
@@ -28,12 +27,6 @@
 #endif
 
 #include <sys/time.h>
-
-#if defined(HAVE_TERMIOS_H)
-# include <termios.h>
-#endif
-
-
 #include <sys/types.h>
 
 #ifdef SYSV
@@ -46,6 +39,12 @@
 # endif
 #endif
 
+
+#ifdef __BEOS__
+/* Prototype for select */
+# include <net/socket.h>
+#endif
+
 #include <sys/file.h>
 
 #ifndef sun
@@ -71,8 +70,7 @@
 #include "slang.h"
 #include "_slang.h"
 
-#define TTY_STDERR_FD 2
-int SLang_TT_Read_FD;
+int SLang_TT_Read_FD = -1;
 int SLang_TT_Baud_Rate;
 
 
@@ -83,6 +81,14 @@ int SLang_TT_Baud_Rate;
 #endif
 
 #ifndef HAVE_TERMIOS_H
+
+# if !defined(CBREAK) && defined(sun)
+#  ifndef BSD_COMP
+#   define BSD_COMP 1
+#  endif
+#  include <sys/ioctl.h>
+# endif
+
 typedef struct
   {
       struct tchars t;
@@ -91,10 +97,11 @@ typedef struct
   }
 TTY_Termio_Type;
 #else
+# include <termios.h>
 typedef struct termios TTY_Termio_Type;
 #endif
 
-TTY_Termio_Type Old_TTY;
+static TTY_Termio_Type Old_TTY;
 
 #ifdef HAVE_TERMIOS_H
 static int Baud_Rates[20] = 
@@ -104,65 +111,101 @@ static int Baud_Rates[20] =
 };
 #endif
 
-/* this next works on ultrix for setting termios */
-#ifdef TCGETS
-# define GET_TERMIOS(fd, x) ioctl(fd, TCGETS, x)
-# define SET_TERMIOS(fd, x) ioctl(fd, TCSETS, x)
+#ifdef HAVE_TERMIOS_H
+# define GET_TERMIOS(fd, x) tcgetattr(fd, x)
+# define SET_TERMIOS(fd, x) tcsetattr(fd, TCSADRAIN, x)
 #else
-# ifndef HAVE_TERMIOS_H
+# ifdef TCGETS
+#  define GET_TERMIOS(fd, x) ioctl(fd, TCGETS, x)
+#  define SET_TERMIOS(fd, x) ioctl(fd, TCSETS, x)
+# else
 #  define X(x,m)  &(((TTY_Termio_Type *)(x))->m)
 #  define GET_TERMIOS(fd, x)	\
-    (ioctl(fd, TIOCGETC, X(x,t)) || \
-     ioctl(fd, TIOCGLTC, X(x,lt)) || \
-     ioctl(fd, TIOCGETP, X(x,s))) ? -1 : 0
+    ((ioctl(fd, TIOCGETC, X(x,t)) || \
+      ioctl(fd, TIOCGLTC, X(x,lt)) || \
+      ioctl(fd, TIOCGETP, X(x,s))) ? -1 : 0)
 #  define SET_TERMIOS(fd, x)	\
-    (ioctl(fd, TIOCSETC, X(x,t)) ||\
-     ioctl(fd, TIOCSLTC, X(x,lt)) || \
-     ioctl(fd, TIOCSETP, X(x,s))) ? -1 : 0
-# else
-#  define GET_TERMIOS(fd, x) tcgetattr(fd, x)
-#  define SET_TERMIOS(fd, x) tcsetattr(fd, TCSAFLUSH, x)
-/* #  define SET_TERMIOS(fd, x) tcsetattr(fd, TCSANOW, x) */
+    ((ioctl(fd, TIOCSETC, X(x,t)) ||\
+      ioctl(fd, TIOCSLTC, X(x,lt)) || \
+      ioctl(fd, TIOCSETP, X(x,s))) ? -1 : 0)
 # endif
 #endif
 
 static int TTY_Inited = 0;
 static int TTY_Open = 0;
 
-#ifdef _POSIX_VDISABLE
-# define NULL_VALUE _POSIX_VDISABLE
+#ifdef ultrix   /* Ultrix gets _POSIX_VDISABLE wrong! */
+# define NULL_VALUE -1
 #else
-# define NULL_VALUE 255
+# ifdef _POSIX_VDISABLE
+#  define NULL_VALUE _POSIX_VDISABLE
+# else
+#  define NULL_VALUE 255
+# endif
 #endif
 
 
 int SLang_init_tty (int abort_char, int no_flow_control, int opost)
 {
    TTY_Termio_Type newtty;
-
-   if (TTY_Inited) return 0;
-   TTY_Open = 0;
-#ifdef O_RDWR
-   if ((SLang_TT_Read_FD = open("/dev/tty", O_RDWR)) >= 0)
+   
+   SLsig_block_signals ();
+   
+   if (TTY_Inited) 
      {
-	TTY_Open = 1;
+	SLsig_unblock_signals ();
+	return 0;
      }
-   else
+   
+   TTY_Open = 0;
+   
+   if ((SLang_TT_Read_FD == -1)
+       || (1 != isatty (SLang_TT_Read_FD)))
+     {
+#ifdef O_RDWR
+# ifndef __BEOS__  /* I have been told that BEOS will HANG if passed /dev/tty */
+	if ((SLang_TT_Read_FD = open("/dev/tty", O_RDWR)) >= 0)
+	  {
+	     TTY_Open = 1;
+	  }
+# endif
 #endif
-     SLang_TT_Read_FD = TTY_STDERR_FD;
-
+	if (TTY_Open == 0)
+	  {
+	     SLang_TT_Read_FD = fileno (stderr);
+	     if (1 != isatty (SLang_TT_Read_FD))
+	       {
+		  SLang_TT_Read_FD = fileno (stdin);
+		  if (1 != isatty (SLang_TT_Read_FD))
+		    {
+		       fprintf (stderr, "Failed to open terminal.");
+		       return -1;
+		    }
+	       }
+	  }
+     }
+   
    SLang_Abort_Char = abort_char;
    
+   /* Some systems may not permit signals to be blocked.  As a result, the 
+    * return code must be checked.
+    */
    while (-1 == GET_TERMIOS(SLang_TT_Read_FD, &Old_TTY))
      {
-	if (errno != EINTR) 
-	  return -1;
+	if (errno != EINTR)
+	  {
+	     SLsig_unblock_signals ();
+	     return -1;
+	  }
      }
    
    while (-1 == GET_TERMIOS(SLang_TT_Read_FD, &newtty))
      {
 	if (errno != EINTR)
-	  return -1;
+	  {
+	     SLsig_unblock_signals ();
+	     return -1;
+	  }
      }
    
 #ifndef HAVE_TERMIOS_H
@@ -216,11 +259,15 @@ int SLang_init_tty (int abort_char, int no_flow_control, int opost)
    
    while (-1 == SET_TERMIOS(SLang_TT_Read_FD, &newtty))
      {
-	if (errno == EINTR)
-	  return -1;
+	if (errno != EINTR)
+	  {
+	     SLsig_unblock_signals ();
+	     return -1;
+	  }
      }
    
    TTY_Inited = 1;
+   SLsig_unblock_signals ();
    return 0;
 }
 
@@ -228,11 +275,17 @@ void SLtty_set_suspend_state (int mode)
 {
    TTY_Termio_Type newtty;
    
-  while ((-1 == GET_TERMIOS (SLang_TT_Read_FD, &newtty))
-	 && (errno == EINTR))
+   SLsig_block_signals ();
+   
+   if (TTY_Inited == 0)
+     {
+	SLsig_unblock_signals ();
+	return;
+     }
+      
+   while ((-1 == GET_TERMIOS (SLang_TT_Read_FD, &newtty))
+	  && (errno == EINTR))
      ;
-	  
-     
    
 #ifndef HAVE_TERMIOS_H
    if (mode == 0) newtty.lt.t_suspc = 255;
@@ -245,11 +298,19 @@ void SLtty_set_suspend_state (int mode)
    while ((-1 == SET_TERMIOS (SLang_TT_Read_FD, &newtty))
 	  && (errno == EINTR))
      ;
+
+   SLsig_unblock_signals ();
 }
 
 void SLang_reset_tty (void)
 {
-   if (TTY_Inited == 0) return;
+   SLsig_block_signals ();
+   
+   if (TTY_Inited == 0)
+     {
+	SLsig_unblock_signals ();
+	return;
+     }
    
    while ((-1 == SET_TERMIOS(SLang_TT_Read_FD, &Old_TTY))
 	  && (errno == EINTR))
@@ -262,28 +323,31 @@ void SLang_reset_tty (void)
 	  ;
 	  
 	TTY_Open = 0;
+	SLang_TT_Read_FD = -1;
      }
    
    TTY_Inited = 0;
+   SLsig_unblock_signals ();
 }
 
-#ifdef __cplusplus
-# define SIGNAL(a,b) signal((a), (SIG_PF)(b))
-#else
-# define SIGNAL signal
-#endif
-
 static void default_sigint (int sig)
 {
+   sig = errno;			       /* use parameter */
+   
    SLKeyBoard_Quit = 1;
    if (SLang_Ignore_User_Abort == 0) SLang_Error = USER_BREAK;
-   SIGNAL (SIGINT, default_sigint);
+   SLsignal_intr (SIGINT, default_sigint);
+   errno = sig;
 }
 
 void SLang_set_abort_signal (void (*hand)(int))
 {
+   int save_errno = errno;
+   
    if (hand == NULL) hand = default_sigint;
-   SIGNAL (SIGINT, hand);
+   SLsignal_intr (SIGINT, hand);
+   
+   errno = save_errno;
 }
 
 #ifndef FD_SET
@@ -296,13 +360,27 @@ typedef int fd_set;
 static fd_set Read_FD_Set;
 
 
+/* HACK: If > 0, use 1/10 seconds.  If < 0, use 1/1000 seconds */
+
 int SLsys_input_pending(int tsecs)
 {
    struct timeval wait;
    long usecs, secs;
 
-   secs = tsecs / 10;
-   usecs = (tsecs % 10) * 100000;
+   if (TTY_Inited == 0) return -1;
+   
+   if (tsecs >= 0)
+     {
+	secs = tsecs / 10;
+	usecs = (tsecs % 10) * 100000;
+     }
+   else
+     {
+	tsecs = -tsecs;
+	secs = tsecs / 1000;
+	usecs = (tsecs % 1000) * 1000;
+     }
+   
    wait.tv_sec = secs;
    wait.tv_usec = usecs;
 
@@ -313,19 +391,74 @@ int SLsys_input_pending(int tsecs)
 }
 
 
+int (*SLang_getkey_intr_hook) (void);
+
+static int handle_interrupt (void)
+{
+   if (SLang_getkey_intr_hook != NULL)
+     {
+	int save_tty_fd = SLang_TT_Read_FD;
+   
+	if (-1 == (*SLang_getkey_intr_hook) ())
+	  return -1;
+	
+	if (save_tty_fd != SLang_TT_Read_FD)
+	  return -1;
+     }
+   
+   return 0;
+}
 
 unsigned int SLsys_getkey (void)
 {
    unsigned char c;
    
-   while ((SLKeyBoard_Quit == 0)
-	  && !SLsys_input_pending(100));
-   
-   while (!SLKeyBoard_Quit
-	  && (-1 == read(SLang_TT_Read_FD, (char *) &c, 1)))
+   if (TTY_Inited == 0)
      {
-	if (errno == EINTR) continue;
-
+	int ic = fgetc (stdin);
+	if (ic == EOF) return SLANG_GETKEY_ERROR;
+	return (unsigned int) ic;
+     }
+   
+   while (1)
+     {
+	int ret;
+	
+	if (SLKeyBoard_Quit) 
+	  return SLang_Abort_Char;
+	
+	if (0 == (ret = SLsys_input_pending (100)))
+	  continue;
+	
+	if (ret != -1)
+	  break;
+	
+	if (SLKeyBoard_Quit) 
+	  return SLang_Abort_Char;
+	
+	if (errno == EINTR)
+	  {
+	     if (-1 == handle_interrupt ())
+	       return SLANG_GETKEY_ERROR;
+	     
+	     continue;
+	  }
+	
+	break;			       /* let read handle it */
+     }
+   
+   while (-1 == read(SLang_TT_Read_FD, (char *) &c, 1))
+     {
+	if (errno == EINTR) 
+	  {
+	     if (-1 == handle_interrupt ())
+	       return SLANG_GETKEY_ERROR;
+	     
+	     if (SLKeyBoard_Quit) 
+	       return SLang_Abort_Char;
+	     
+	     continue;
+	  }
 #ifdef EAGAIN
 	if (errno == EAGAIN) 
 	  {
@@ -340,11 +473,15 @@ unsigned int SLsys_getkey (void)
 	     continue;
 	  }
 #endif
-	return 0xFFFF;
+#ifdef EIO
+	if (errno == EIO)
+	  {
+	     SLang_exit_error ("SLsys_getkey: EIO error.");
+	  }
+#endif
+	return SLANG_GETKEY_ERROR;
      }
 
-   /* only way for keyboard quit to be non zero is if ^G recived and sigint processed */
-   if (SLKeyBoard_Quit) c = SLang_Abort_Char;
-   SLKeyBoard_Quit = 0;
    return((unsigned int) c);
 }
+
