@@ -5,11 +5,13 @@
  * License or the Perl Artistic License.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <string.h>
 
+#include <stdlib.h>
+
 #ifdef __GO32__
 # undef msdos
 #endif
@@ -37,7 +39,9 @@
 # include <os2.h>
 #else
 # if defined (__EMX__)		/* EMX video does both DOS & OS/2 */
-#  define EMX_VIDEO
+#  ifndef EMX_VIDEO
+#   define EMX_VIDEO
+#  endif
 #  include <sys/video.h>
 # endif
 #endif
@@ -45,6 +49,10 @@
 #include <dos.h>
 #include "slang.h"
 
+#ifdef GO32_VIDEO
+# define HAS_SAVE_SCREEN
+#endif
+
 /* ------------------------- global variables ------------------------- */
 
 #ifdef WIN32
@@ -191,6 +199,43 @@ void ScreenUpdate (unsigned char *src)
 }
 #endif	/* __WATCOMC__ */
 
+#ifdef HAS_SAVE_SCREEN
+static void *Saved_Screen_Buffer;
+static int Saved_Cursor_Row;
+
+static void save_screen (void)
+{
+   int row, col;
+   
+   if (Saved_Screen_Buffer != NULL)
+     {
+	SLFREE (Saved_Screen_Buffer);
+	Saved_Screen_Buffer = NULL;
+     }
+#ifdef GO32_VIDEO   
+   Saved_Screen_Buffer = SLMALLOC (sizeof (short) * 
+				   ScreenCols () * ScreenRows ());
+
+   if (Saved_Screen_Buffer == NULL)
+     return;
+   
+   ScreenRetrieve (Saved_Screen_Buffer);
+   ScreenGetCursor (&row, &col);
+   Saved_Cursor_Row = row;
+#endif
+   
+}
+
+static void restore_screen (void)
+{
+   if (Saved_Screen_Buffer == NULL) return;
+#ifdef GO32_VIDEO
+   ScreenUpdate (Saved_Screen_Buffer);
+   SLtt_goto_rc (Saved_Cursor_Row, 0);
+#endif
+   
+}
+#endif				       /* HAS_SAVE_SCREEN */
 /*----------------------------------------------------------------------*\
  * Function:	void SLtt_write_string (char *str);
  *
@@ -346,9 +391,19 @@ static void slvid_deleol (int x)
 #if defined (EMX_VIDEO)
 static void slvid_deleol (int x)
 {
+   unsigned char *p, *pmax;
    int w = mkSpaceChar ();
    int count = SLtt_Screen_Cols - x;
-   memset (Line_Buffer, w, 2*count);
+   
+   p = Line_Buffer;
+   pmax = p + 2 * count;
+   
+   while (p < pmax)
+     {
+	*p++ = (unsigned char) w;
+	*p++ = (unsigned char) (w >> 8);
+     }
+
    v_putline (Line_Buffer, x, Cursor_Row, count);
 }
 #endif	/* EMX_VIDEO */
@@ -1079,6 +1134,9 @@ void SLtt_smart_puts (unsigned short *new_string,
 void SLtt_reset_video (void)
 {
    SLtt_goto_rc (SLtt_Screen_Rows - 1, 0);
+#ifdef HAS_SAVE_SCREEN
+   restore_screen ();
+#endif
    Attribute_Byte = 0x07;
    Current_Color = JNO_COLOR;
    SLtt_del_eol ();
@@ -1335,6 +1393,10 @@ void SLtt_get_terminfo (void)
    SLtt_Screen_Rows = csbiInfo.dwSize.Y;
    SLtt_Screen_Cols = csbiInfo.dwSize.X;
 #endif
+#ifdef GO32_VIDEO
+   SLtt_Screen_Rows = ScreenRows ();
+   SLtt_Screen_Cols = ScreenCols ();
+#endif
 }
 
 /*----------------------------------------------------------------------*\
@@ -1346,6 +1408,10 @@ void SLtt_init_video (void)
    int OldCol, OldRow;
 #endif
    
+#ifdef HAS_SAVE_SCREEN
+   save_screen ();
+#endif
+
    Cursor_Row = Cursor_Col = 0;
    
 #if defined (EMX_VIDEO)
@@ -1430,7 +1496,7 @@ void SLtt_init_video (void)
 	int86 (0x10, &r, &r);
 	Blink_Killed = 1;
      }
-#  else	/* __GO32__ or __WATCOMC__ */
+#  else	/* (__GO32__ or __WATCOMC__ */
      {
 	unsigned char *p = (unsigned char far *) 0x00400049L;
 	if (*p == 7)
@@ -1513,4 +1579,12 @@ int SLtt_set_cursor_visibility (int show)
    (void) show;
    return -1;
 }
+
+void SLtt_set_mono (int obj_unused, char *unused, SLtt_Char_Type c_unused)
+{
+   (void) obj_unused;
+   (void) unused;
+   (void) c_unused;
+}
+
 /* /////////////////////// end of file (c source) ///////////////////// */
