@@ -29,6 +29,8 @@
 #include "slang.h"
 #include "_slang.h"
 
+/* If this function returns -1, ^Y will be added to input buffer. */
+int (*SLtty_VMS_Ctrl_Y_Hook) (void);
 
 typedef struct {                /* I/O status block     */
         short i_cond;           /* Condition value      */
@@ -129,10 +131,15 @@ static int getkey_ast(int not_used)
 	     SLKeyBoard_Quit = 1;
 	  }
 	
-	if (SLang_Input_Buffer_Len < MAX_INPUT_BUFFER_LEN - 3) 
-	  SLang_Input_Buffer[SLang_Input_Buffer_Len++] = c;
-	
+	if ((c != 0x19)		       /* ^Y */
+	    || (SLtty_VMS_Ctrl_Y_Hook == NULL)
+	    || (-1 == (*SLtty_VMS_Ctrl_Y_Hook) ()))
+	  {
+	     if (SLang_Input_Buffer_Len < MAX_INPUT_BUFFER_LEN - 3) 
+	       SLang_Input_Buffer[SLang_Input_Buffer_Len++] = c;
+	  }
      }
+
    if (Waiting_For_Ast)  sys$setef (Ast_Fired_Event_Flag);
    Waiting_For_Ast = 0;
    vms_que_key_ast();
@@ -371,3 +378,4 @@ int SLsys_input_pending(int tsecs)
 void SLang_set_abort_signal (void (*f)(int))
 {
 }
+
